// FILE: ./assets/js/services/LocalPlateDataService.js
function LocalPlateDataService(){
    let cache = JSON.parse(localStorage.getItem('savedCons')) || [];
    
    const _findPlateFunction = function(plate) {
        return sc => 
            // sc.CDFILCENTPME == plate.CDFILCENTPME &&
            // sc.CDTIPOCARD == plate.CDTIPOCARD &&
            // sc.NRCARDPME == plate.NRCARDPME &&
            // sc.CDDIETPAD == plate.CDDIETPAD &&
            sc.CDFILIAL == plate.CDFILIAL &&
            sc.CDSERVFILI == plate.CDSERVFILI &&
            sc.NRCARDAPIO == plate.NRCARDAPIO &&
            sc.NRSEQPRATCAR == plate.NRSEQPRATCAR &&
            sc.CDFILAUXPRAT == plate.CDFILAUXPRAT &&
            sc.CDPRATO == plate.CDPRATO &&
            sc.DTCARDAPIO == plate.DTCARDAPIO &&
            sc.CDTIPOPRAT == plate.CDTIPOPRAT &&
            sc.NRORG == plate.NRORG;
    };
    
    const findPlate = function(plate) {
        const findKeys = _findPlateFunction(plate);
        return cache.find(findKeys);
    };
    
    const findPlateIndex = function(plate) {
        const findKeys = _findPlateFunction(plate);
        return cache.findIndex(findKeys);
    };
    
    const hasPlate = function(plate) {
        return !!findPlate(plate);
    };
    
    this.plateIsBookmarked = function(plate) {
        const cachePlate = findPlate(plate);
        return cachePlate && cachePlate.IDCONSUMO;
    };
    
    this.isPlateRated = function(plate){
        const cachePlate = findPlate(plate);
        return cachePlate && cachePlate.VRAVALIACAOPRATO;
    };
    
    const update = function(plate) {
        const plateIndex = findPlateIndex(plate);
        cache[plateIndex] = plate;
        setLocalVar('savedCons', cache);
        console.log('update called with params:', plate);
    };
    
    const insert = function(plate){
        cache.push(plate);
        setLocalVar('savedCons', cache);
        console.log('insert called with params:', plate);
    };
    
    this.saveRating = function(plate) {
        let cachePlate = findPlate(plate);
        
        if(!cachePlate){
            insert(plate);
            cachePlate = plate;
        }
        
        cachePlate.DSAVALICAO = plate.DSAVALICAO;
        cachePlate.VRAVALIACAOPRATO = plate.VRAVALIACAOPRATO;
        
        update(cachePlate);
    };
    
    this.saveBookmark = function(plate) {
        let cachePlate = findPlate(plate);
        
        if(!cachePlate){
            insert(plate);
            cachePlate = plate;
        }
        
        cachePlate.IDCONSUMO = plate.IDCONSUMO;
        cachePlate.NMUSUARIO = plate.NMUSUARIO;
        cachePlate.HRCONSUMO = plate.HRCONSUMO;
        cachePlate.NRAVALIACAOPRATO = plate.NRAVALIACAOPRATO;
        
        update(cachePlate);
    };
    
    this.findPlate = findPlate;
    this.hasPlate = hasPlate;
    this.insert = insert;
    this.update = update;
    this.reloadCache = () => {
        cache = JSON.parse(localStorage.getItem('savedCons')) || [];
    };
}


Configuration(function (ContextRegister) {
    ContextRegister.register('LocalPlateDataService', LocalPlateDataService);
});

// FILE: ./assets/js/controllers/AlterarSenhaController.js
function AlterarSenhaController(ScreenService, MessageService, General) {
    
    this.changePassword = function(widget) {
        if (widget.isValid()) {
            var currentRow = widget.currentRow;
            if (currentRow.newPw == currentRow.confirmPw) {
                var pwData = {
                    "OLD_PW": currentRow.currentPw,
                    "NEW_PW": currentRow.newPw
                };
                General.callRestEngine("/changePassword", "Row", pwData).then(function(){
                    MessageService.showMessageByCode('MSG_SUCCESS_RECOVER');
                    
                    widget.newRow();
                    
                    ScreenService.openWindow('product#home');
                }, function(response){
                    var errorMsg = "MSG_INVALID_PASSWORD";
                    if (response.data && response.data.error) {
                        errorMsg = response.data.error;
                    }
        
                    MessageService.showMessageByCode(errorMsg);
                });
            } else {
                MessageService.showMessageByCode('MSG_VALID_PWRECOVER');
                currentRow.newPw = "";
                currentRow.confirmPw = "";
            }
        }
    };
    
    this.cancel = function(widget){
        widget.newRow();
        ScreenService.openWindow('product#home');
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register('AlterarSenhaController', AlterarSenhaController);
});

// FILE: ./assets/js/controllers/CardapioController.js
function MyMenuCardapioController(ScreenService, templateManager, restEngine, LocalPlateDataService, MessageService) {
    this.doNothing = function () {};
    this.onEnter = function (widget) {};

    function saveCons(data) {
        return restEngine.request("/consSave", {
            requestType: "Row",
            row: data
        });
    }

    function removeCons(data) {
        return restEngine.request("/consRemove", {
            requestType: "Row",
            row: data
        });
    }

    function saveSrvCons(data) {
        return restEngine.request("/consSrvSave", {
            requestType: "Row",
            row: data
        });
    }

    function save(data) {
        return restEngine.request("/rateSave", {
            requestType: "Row",
            row: data
        });
    }

    this.resetBookMarking = function(widget) {
        widget.bookMarking = [];
        widget.reload();
    };
    
    this.backToLastWidget = function (widget) {
        let widgets = widget.container.widgets;
        let acwidget = widgets.indexOf(widget);

        widget.isVisible = false;

        let widgetToGo = widgets[acwidget - 1];

        widgetToGo.container.label = widgetToGo.servdata ? widgetToGo.servdata[widgetToGo.label] : null || widgetToGo.currentRow.servdata ? widgetToGo.currentRow.servdata[widgetToGo.label] : widgetToGo.label;

        widgetToGo.isVisible = true;
        widgetToGo.activate();
    };

    this.saveRateData = function () {

        let widget = templateManager.container.getActiveWidget();
        let dtfil = widget.dataSourceFilter.find(f => f.name === 'DTCARDAPIO').value;
        dtfil = dtfil === "@moment().format('DD/MM/YYYY')@" ? moment() : moment(dtfil, 'DD/MM/YYYY');

        let plate = widget.plateToRate;
        plate.NRUSUARIOAPP = getLocalVar('userData').NRUSUARIOAPP;
        plate.NRORG = getLocalVar('userData').NRORG;
        plate.VRAVALIACAOPRATO = plate.newAv;
        plate.DTCARDAPIO = dtfil.format('DD/MM/YYYY');

        save(plate).then(response => {
            if (getLocalVar('userData').isAnon && getLocalVar('userData').isAnon === true) {
                if (getLocalVar('lastRate')) {
                    let list = getLocalVar('lastRate');
                    let last = {
                        'CDPRATO': plate.CDPRATO,
                        'NRCARDAPIO': plate.NRCARDAPIO,
                        'CDTIPOPRAT': plate.CDTIPOPRAT,
                        'DATE': dtfil.format('DD/MM/YYYY')
                    };
                    list.push(last);
                    setLocalVar('lastRate', list);
                } else {
                    let last = {
                        'CDPRATO': plate.CDPRATO,
                        'NRCARDAPIO': plate.NRCARDAPIO,
                        'CDTIPOPRAT': plate.CDTIPOPRAT,
                        'DATE': dtfil.format('DD/MM/YYYY')
                    };
                    setLocalVar('lastRate', [last]);
                }
            }
            widget.reload();
            MessageService.showMessageByCode('MSG_RATED_WITH_SUCCESS', null, {type: 'success'});
        }).catch(error => {
            ScreenService.showMessage(error || error.message || 'Erro!');
        });
        widget.plateToRate = null;
    };

    this.resetTitle = () => templateManager.container.label = "CARDÁPIO";

    function getFormatedPlateForCons(plate) {
        return {
            'CDPRATO': plate.CDPRATO,
            'CDFILCENTPME': plate.CDFILCENTPME,
            'CDTIPOCARD': plate.CDTIPOCARD,
            'NRCARDPME': plate.NRCARDPME,
            'CDDIETPAD': plate.CDDIETPAD,
            'CDFILIAL': plate.CDFILIAL,
            'CDSERVFILI': plate.CDSERVFILI,
            'NRCARDAPIO': plate.NRCARDAPIO,
            'NRSEQPRATCAR': plate.NRSEQPRATCAR,
            'CDFILAUXPRAT': plate.CDFILAUXPRAT,
            'DTCARDAPIO': plate.DTCARDAPIO,
            'CDTIPOPRAT': plate.CDTIPOPRAT,
            'IDCONSUMO': plate.IDCONSUMO,
            'NMUSUARIO': plate.NMUSUARIO,
            'HRCONSUMO': plate.HRCONSUMO,
            'NRORG': plate.NRORG,
            
            IDOPCCNFMES: plate.IDOPCCNFMES,
            IDCONCNFMES: plate.IDCONCNFMES
        };
    }

    function getFormatedServiceForCons(service) {
        return {
            'IDCONSUMO': service.IDCONSUMO,
            'DTCARDAPIO': service.DTCARDAPIO,
            'CDFILIAL': service.CDFILIAL,
            'CDSERVFILI': service.CDSERVFILI,
            'CDTIPOCARD': service.CDTIPOCARD,
            'NMUSUARIO': service.NMUSUARIO,
            'HRCONSUMO': service.HRCONSUMO,
            'NRORG': service.NRORG
        };
    }

    function editLocalConsData(plate) {
        let savedCons = getLocalVar('savedCons');
        if (savedCons) {
            let consToUpdate = savedCons.find(sc => sc.CDPRATO == plate.CDPRATO && sc.CDFILCENTPME == plate.CDFILCENTPME && sc.CDTIPOCARD == plate.CDTIPOCARD && sc.NRCARDPME == plate.NRCARDPME && sc.CDDIETPAD == plate.CDDIETPAD && sc.CDFILIAL == plate.CDFILIAL && sc.CDSERVFILI == plate.CDSERVFILI && sc.NRCARDAPIO == plate.NRCARDAPIO && sc.NRSEQPRATCAR == plate.NRSEQPRATCAR && sc.CDFILAUXPRAT == plate.CDFILAUXPRAT && sc.CDPRATO == plate.CDPRATO && sc.DTCARDAPIO == plate.DTCARDAPIO && sc.CDTIPOPRAT == plate.CDTIPOPRAT && sc.NRORG == plate.NRORG);
            if (consToUpdate) {
                consToUpdate.IDCONSUMO = plate.IDCONSUMO;
                consToUpdate.NMUSUARIO = plate.NMUSUARIO;
                consToUpdate.HRCONSUMO = plate.HRCONSUMO;
            } else {
                savedCons.push(getFormatedPlateForCons(plate));
            }
        } else {
            savedCons = [];
            savedCons.push(getFormatedPlateForCons(plate));
        }
        setLocalVar('savedCons', savedCons);
    }

    function editLocalSrvConsData(service) {
        let savedCons = getLocalVar('savedSrvCons');
        if (savedCons) {
            let consToUpdate = savedCons.find(sc => sc.CDFILIAL == service.CDFILIAL && sc.CDSERVFILI == service.CDSERVFILI && sc.CDTIPOCARD == service.CDTIPOCARD && sc.DTCARDAPIO == service.DTCARDAPIO && sc.NRORG == service.NRORG);
            if (consToUpdate) {
                consToUpdate.IDCONSUMO = service.IDCONSUMO;
                consToUpdate.NMUSUARIO = service.NMUSUARIO;
                consToUpdate.HRCONSUMO = service.HRCONSUMO;
            } else {
                savedCons.push(getFormatedServiceForCons(service));
            }
        } else {
            savedCons = [];
            savedCons.push(getFormatedServiceForCons(service));
        }
        setLocalVar('savedSrvCons', savedCons);
    }

    function saveIDCONCNFMES(plate) {
        let savedIDCONCNFMES = getLocalVar('IDCONCNFMESLIST');
        if (plate.IDCONCNFMES == 'S') {
            if (savedIDCONCNFMES) {
                savedIDCONCNFMES = savedIDCONCNFMES.filter(sicm => !(plate.CDTIPOPRAT == sicm.CDTIPOPRAT && plate.NRCARDAPIO == sicm.NRCARDAPIO && plate.CDPRATO == sicm.CDPRATO));
                if (plate.IDCONSUMO == 'S') {
                    savedIDCONCNFMES.push(plate);
                }
            } else {
                savedIDCONCNFMES = [];
                savedIDCONCNFMES.push(plate);
            }
        }
        setLocalVar('IDCONCNFMESLIST', savedIDCONCNFMES);
    }

    this.deleteConsData = function () {
        let widget = templateManager.container.getActiveWidget();
        const isAnon = getLocalVar('userData').isAnon;
        let plates = [];
        if(isAnon){
            let cachedBookmarks = getLocalVar("savedCons");
            const servFili = widget.currentRow.servdata.CDSERVFILI;
            cachedBookmarks = cachedBookmarks.filter((local) => local.CDSERVFILI = servFili);
            cachedBookmarks.forEach((local) => {
                const plate = widget.dataSource.data.filter((data) => {
                    const dtCard = data.DTCARDAPIO === "@moment().format('DD/MM/YYYY')@" ? moment().format('DD/MM/YYYY') : data.DTCARDAPIO;
                    if(data.CDPRATO == local.CDPRATO && local.DTCARDAPIO == dtCard) {
                        return data;
                    }
                });
                if(plate[0]) plates.push(plate[0]);
            });
        } else {
            plates = widget.dataSource.data.filter((data) => data.IDCONSUMO == 'S');
        }
        
        plates.forEach((plate, key) => {
            plate.IDCONSUMO = 'N';
            plate.NRORG = getLocalVar('userData').NRORG;
            plate.isAnon = isAnon;
            plate.NRUSUARIOAPP = getLocalVar('userData').NRUSUARIOAPP;
            plate.HRCONSUMO = plate.isAnon ? null : plate.HRCONSUMO.replace(':', '');
            plate.DTCARDAPIO = plate.DTCARDAPIO === "@moment().format('DD/MM/YYYY')@" ? moment().format('DD/MM/YYYY') : plate.DTCARDAPIO;
            if (plate.isAnon) {
                plate.NRUSUARIOAPP = null;
                LocalPlateDataService.reloadCache();
                const localPlate = plate.IDCONSUMO === 'N' ?  LocalPlateDataService.findPlate(plate) : plate;
                plates[key] = Object.assign(localPlate, plate);
                editLocalConsData(plate);
            }
            saveIDCONCNFMES(plate);
        });
        
        saveCons(plates).then(() => {
            ScreenService.notificationMessage('Reserva cancelada!', 'success');
            widget.reload();
        });
    };

    this.saveConsData = function () {
        let widget = templateManager.container.getActiveWidget();
        let plates = widget.bookMarking;

        if(plates && plates.length > 0) {
            const nmTpPratoCons = this.checkCons(widget);
            if(nmTpPratoCons == '') {
                plates.forEach((plate) => {
                    plate.NRORG = getLocalVar('userData').NRORG;
                    plate.isAnon = getLocalVar('userData').isAnon;
                    plate.NRUSUARIOAPP = getLocalVar('userData').NRUSUARIOAPP;
                    plate.HRCONSUMO = widget.dataBookmarking.HRCONSUMO.replace(':', '');
                    plate.DTCARDAPIO = plate.DTCARDAPIO === "@moment().format('DD/MM/YYYY')@" ? moment().format('DD/MM/YYYY') : plate.DTCARDAPIO;
                    if (plate.isAnon) {
                        plate.NRUSUARIOAPP = null;
                        LocalPlateDataService.reloadCache();
                        const localPlate = plate.IDCONSUMO === 'N' ?  LocalPlateDataService.findPlate(plate) : plate;
                        plate = Object.assign(localPlate, plate);
                        editLocalConsData(plate);
                    }
                    saveIDCONCNFMES(plate);
                });
                saveCons(plates).then(() => {
                    ScreenService.notificationMessage('Reserva confirmada!', 'success');
                    widget.bookMarking = [];
                    widget.doReservation = false;
                    widget.reload();
                });
            } else {
                ScreenService.showMessage(`Selecione pelo menos um prato do(s) tipo(s): ${nmTpPratoCons}`, 'error');
                widget.doReservation = false;
            }
            
        }
    };

    this.checkCons = function(widget) {
        let nmTpPratoCons = '';
        let cdtpPrato = widget.dataSource.data.map((cdprato) => cdprato.CDTIPOPRAT);
        cdtpPrato = [...new Set(cdtpPrato)];
        cdtpPrato.forEach((cdprato) => {
            const arrObject = widget.dataSource.data.filter((data) => data.CDTIPOPRAT == cdprato && data.IDCONCNFMES == 'S');
            if(arrObject.length > 0) {
                const selectedPratCons = arrObject.filter((object) => object.IDCONSUMO == 'S');
                if (selectedPratCons.length == 0) {
                    nmTpPratoCons = arrObject[0].NMTIPOPRAT;
                    return;
                }
            }
        });
        return nmTpPratoCons;
    };

    this.saveSrvConsData = function () {
        let widget = templateManager.container.getActiveWidget();
        let servdata = widget.dataBookmarking;

        let dtfil = widget.dataSourceFilter.find(f => f.name === 'DTCARDAPIO').value;
        dtfil = dtfil === "@moment().format('DD/MM/YYYY')@" ? moment() : moment(dtfil, 'DD/MM/YYYY');
        servdata.DTCARDAPIO = dtfil.format('DD/MM/YYYY');

        servdata.NRORG = getLocalVar('userData').NRORG;
        servdata.isAnon = getLocalVar('userData').isAnon;
        servdata.NRUSUARIOAPP = getLocalVar('userData').NRUSUARIOAPP;
        if (servdata.isAnon) {
            servdata.NRUSUARIOAPP = null;
            editLocalSrvConsData(servdata);
        }
        saveSrvCons(servdata).then(() => {
            widget.dataBookmarking = null;
            widget.reload();
        });
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register('MyMenuCardapioController', MyMenuCardapioController);
});

// FILE: ./assets/js/controllers/CriarContaController.js
function CriarContaController(ScreenService, General, engine, templateManager, MessageService) {
    
    this.onEnter = function(widget){
        widget.newRow();
        this.setTOSUrl(widget);
        widget.currentRow.tos_confirmation = 'N';
    };
    
    this.setTOSUrl = function(widget){
        const tosField = widget.fields.find(f => f.name === 'tos_confirmation');
        const TOSWidget = templateManager.container.getWidget("tosIframe");
        if(tosField){
            const tosURL = getLocalVar('tosURL');
            tosField.tosUrl = tosURL;
            TOSWidget.sourceUrl = tosURL;
        }

    };

    this.submit = function(widget) {
        const hasAcceptedTermsOfService = widget.currentRow.tos_confirmation === 'Y';
        if (widget.isValid() && hasAcceptedTermsOfService) {
            var currentRow = widget.currentRow;
            
            if (currentRow.email == currentRow.confEmail && currentRow.email !== 'anonimo@anonimo.com') {
                if (currentRow.password == currentRow.confPw) {
                    General.callRestEngine("/createAccount", "Row", currentRow).then(function(){
                        MessageService.showMessageByCode('MSG_SUBMIT_ACCOUNT_SUCCESS');
                        ScreenService.openWindow('product#login');
                    }, function(response){
                        var errorMsg = "Erro";
                        if (response.data && response.data.error) {
                            errorMsg = response.data.error;
                        } else if(response.error){
                            errorMsg = response.error;
                        }
            
                        ScreenService.showMessage(ScreenService.i18n(errorMsg), 'E');
                    });
                }
                else {
                    MessageService.showMessageByCode('MSG_VALID_PWACCOUNT_SUBMIT');
                    currentRow.newPw = "";
                    currentRow.confirmPw = "";
                }
            } else {
                MessageService.showMessageByCode('MSG_VALID_EMAIL_SUBMIT');
                currentRow.newPw = "";
                currentRow.confirmPw = "";
            }
        } else if(!hasAcceptedTermsOfService) {
            MessageService.showMessageByCode('MSG_ACCEPT_SERVICE_TERM');
        }
    };
    
    this.cancel = function(widget){
        ScreenService.openWindow('product#login');
    };

    this.showRemoveAccountConfirmationDialog = function(){
        MessageService.confirmMessageByCode('MSG_DELETE_ACCOUNT', null, {
            type: 'question',
            success: () => {
                General.callRestEngine("/removeAccount", "Row", {}).then(() => {
                    engine.forceLogOut();             
                });
            }
        });
    };

    this.showTermsOfServiceWidget = function(event){
        const clickedElement = event.path[0];
        const hasTOS = clickedElement.getAttribute("terms-of-service");
        // console.log(templateManager);
        if(hasTOS === ""){
            const TOSWidget = templateManager.container.getWidget("tosIframe");
            ScreenService.openPopup(TOSWidget);
            console.log("TOIS");
        }
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register('CriarContaController', CriarContaController);
});

// FILE: ./assets/js/controllers/FaleConoscoController.js
function FaleConoscoController(ScreenService, MessageService, General){
    this.onEnter = function(widget){
        widget.newRow();
    };
    
    this.submit = function(widget){
        if (widget.isValid()) {
            General.callRestEngine("/faleConosco/save", "Row", widget.currentRow).then(
                function(){
                    MessageService.showMessageByCode('MSG_SENT_MSG_WITH_SUCCESS');
                    widget.newRow();
                }, function(){
                    MessageService.showMessageByCode('MSG_SENT_MSG_ERROR');
                }
            );
        }
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register('FaleConoscoController', FaleConoscoController);
});

// FILE: ./assets/js/controllers/HomeController.js
function HomeController(ScreenService, General, templateManager){
    this.changePasswd = function(){
        ScreenService.openWindow("product#alterar_senha");
    };
    this.restarWidgets = container => {
        container.widgets.forEach(widget => {
            widget.isVisible = false;
        });
        let DTCARDAPIO = container.widgets[0].dataSourceFilter.find(dsf=>dsf.name === 'DTCARDAPIO');
        if(DTCARDAPIO){
            DTCARDAPIO.value = "@moment().format('DD/MM/YYYY')@";
        }
        container.widgets[0].activate();
        container.widgets[0].isVisible = true;
        container.widgets[0].reload();
    };

    this.onContainerInit = function(container){
        
    };

    this.setRemoveAccountOptionVisibility = function(homeContainer){
        const homeWidget = homeContainer.widgets.find(widget => widget.name === 'mainWidget');
        const removeAccountAction = homeWidget.actions.find(action => action.name === 'removeAccount');
        const userData = getLocalVar('userData');
        removeAccountAction.isVisible = !userData.isAnon;
        
        const logoutAction = homeWidget.actions.find(action => action.name === 'logout');
        logoutAction.isVisible = getLocalVar("showLogout") == false ? getLocalVar("showLogout") : true;
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register('HomeController', HomeController);
});

// FILE: ./assets/js/controllers/LoginController.js
function LoginController(ScreenService, MessageService, General, engine, ApplicationContext) {
    var self = this;

    const updateToken = (httpRequestResponse) => {
        if(httpRequestResponse.dataset){
            const token = httpRequestResponse.dataset.JWT[0];
            setLocalVar('token', token);
        }
    };


    const setLoginWithAppleButtonVisibility = (widget) => {
        const appleLoginButton = widget.getField('apple_login_button');
        const isIOS = window.cordova && window.cordova.platformId === 'ios';
        const IOSVersion = window.device && parseFloat(window.device.version);
        appleLoginButton.isVisible = isIOS && IOSVersion >= 13;
    };
    
    this.onEnter = function (widget) {
        var url_string = window.location.href;
        var url = new URL(url_string);
        var cript = url.hash.split("params=")[1];
        if(cript) {
            const dns = url.origin + url.pathname.split("/mobile/")[0];
            const newCript = cript.replaceAll("%2F", "/"); 
            ApplicationContext.QRCodeController.qrCodeWeb([dns, newCript]).then(() => {
                this.loginAnonimous();
                setLocalVar("showLogout", false);
            });
        } else {
            widget.container.imageSource = getLocalVar('qrCodeURL') ? window.fileServerPath + getLocalVar('qrCodeURL') : widget.container.imageSource;
            setLoginWithAppleButtonVisibility(widget);
            var applyVisibility = function (hadValidQRCode) {
                var visibleGroup = (hadValidQRCode) ? "login" : "default";
    
                widget.getField('scan').isVisible = !hadValidQRCode;
                widget.fieldGroups.forEach(function (fieldGroup) {
                    fieldGroup.isVisible = (visibleGroup == fieldGroup.name);
                });
            };
    
            if (getLocalVar('validQRCode') && getLocalVar('qrCodeData')) {
                var hadValidQRCode;
    
                General.callRestEngine("/validateQrCode", "Row", getLocalVar('qrCodeData')).then(
                    function (response) {
                        hadValidQRCode = true;
                        updateToken(response);
                    },
                    function (response) {
                        updateToken(response);
                        hadValidQRCode = false;
                    }
                ).finally(function () {
                    applyVisibility(hadValidQRCode);
                });
            } else {
                applyVisibility(false);
            }
            widget.getField('anonimo').isVisible = getLocalVar('IDPERANON') && getLocalVar('IDPERANON') === 'S';
        }
    };

    this.readQrCode = function (widget, qrCodeData) {
        ScreenService.openWindow('product#qrcode');
    };

    this.validateLogin = function (widget) {
        if (widget.isValid()) {
            var userData = {
                'USER': widget.currentRow.email,
                'PW': widget.currentRow.senha
            };
            General.callRestEngine("/login", "Row", userData).then(
                function (response) {
                    updateToken(response);
                    setLocalVar('userData', response.dataset.user);
                    let fcmToken = localStorage.getItem('fcmToken');
                    engine.sendFCMTokenSaveRequest(fcmToken, true).catch(() => engine.sendFCMTokenSaveRequest(fcmToken, false));
                    ScreenService.openWindow('product#home');
                },
                function (response) {
                    var errorMsg = "Erro";
                    if (response && response.error) {
                        errorMsg = response.error;
                    }

                    ScreenService.showMessage(ScreenService.i18n(errorMsg), 'E');
                }
            );
        }
    };

    this.isLoggedWithFacebook = () => {
        let fbExpirationTime = localStorage.getItem("fbExpirationTime");
        if (!fbExpirationTime) {
            return false;
        }
        let expirationDate = new Date();
        expirationDate.setSeconds(fbExpirationTime);
        if ((new Date()) > expirationDate) {
            return false;
        }
        return true;
    };

    this._requestFacebookLogin = () => {
        return new Promise((resolve, reject) => {
            facebookConnectPlugin.login(["public_profile", "email"],
                data => resolve(data),
                error => reject(error)
            );
        });
    };

    this._getUserFacebookData = (userID) => {
        return new Promise((resolve, reject) => {
            facebookConnectPlugin.api(userID + "?fields=first_name,last_name,email", ["public_profile", "email"],
                result => resolve(result),
                error => reject(error)
            );
        });
    };

    /**
     * TODO: document this function, handle errors properly
     */
    this.loginWithFacebook = () => {
        if ('facebookConnectPlugin' in window) {
            this._requestFacebookLogin().then((authData) => {
                let loginData = {};
                let expirationDate = new Date();
                expirationDate.setSeconds(authData.authResponse.expiresIn);
                loginData.accessToken = authData.authResponse.accessToken;
                loginData.expiresIn = moment(expirationDate).format("DD/MM/YYYY");
                loginData.userID = authData.authResponse.userID;
                this._getUserFacebookData(loginData.userID).then((userData) => {
                    Object.assign(loginData, userData);
                    General.callRestEngine("/facebookAuth", "Row", loginData).then(response => {
                        updateToken(response);
                        response.dataset.user.isFacebookConnected = true;
                        setLocalVar('userData', response.dataset.user);
                        localStorage.setItem('fbExpirationTime', loginData.expiresIn);
                        let fcmToken = localStorage.getItem('fcmToken');
                        engine.sendFCMTokenSaveRequest(fcmToken, true).catch(() => engine.sendFCMTokenSaveRequest(fcmToken, false));
                        ScreenService.openWindow('product#home');
                    }, response => {
                        var errorMsg = "Erro";
                        if (response && response.error) {
                            errorMsg = response.error;
                        }
                        ScreenService.showMessage(ScreenService.i18n(errorMsg), 'E');
                    });
                });

            }).catch(err => console.log(err));
        }
    };

    this.loginAnonimous = () => {
        General.callRestEngine("/anonimousAccount", "Row", {}).then(function (response) {
            updateToken(response);
            setLocalVar('userData', response.dataset.user);
            let fcmToken = localStorage.getItem('fcmToken');
            engine.sendFCMTokenSaveRequest(fcmToken, true).catch(() => engine.sendFCMTokenSaveRequest(fcmToken, false));
            ScreenService.openWindow('product#home');
        }, function (response) {
            var errorMsg = "Erro";
            if (response.data && response.data.error) {
                errorMsg = response.data.error;
            }

            ScreenService.showMessage(ScreenService.i18n(errorMsg), 'E');
        });
    };

    const _getAppleIDUserData = () => {
        return new Promise((resolve, reject) => {
            window.cordova.plugins.SignInWithApple.signin(
                { requestedScopes: [0, 1] },
                function (succ) {
                    console.log(succ);
                    resolve(succ);
                },
                function (err) {
                    console.log(JSON.stringify(err));
                    reject(err);
                }
            );
        });
    };

    this.loginWithApple = () => {
        //TODO: implement actual apple signin
        // this.loginAnonimous()
        _getAppleIDUserData().then((userData) => {
            const {fullName, user, email, identityToken} = userData;
            
            const authData = {
                userID: user,
                email,
                first_name: fullName.givenName,
                last_name: fullName.familyName,
                accessToken: identityToken,
                expiresIn: null
            };

            General.callRestEngine("/facebookAuth", "Row", authData).then(response => {
                updateToken(response);
                response.dataset.user.isFacebookConnected = true;
                setLocalVar('userData', response.dataset.user);
                // localStorage.setItem('fbExpirationTime', loginData.expiresIn);
                const fcmToken = localStorage.getItem('fcmToken');
                engine.sendFCMTokenSaveRequest(fcmToken, true).catch(() => engine.sendFCMTokenSaveRequest(fcmToken, false));
                ScreenService.openWindow('product#home');
            });
            
        }).catch(() => {
            MessageService.showMessageByCode("MSG_DENIED_ACCESS_WITH_APPLEID");
        });
        
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register('LoginController', LoginController);
});

// FILE: ./assets/js/controllers/MyMenuInformativosController.js
function MyMenuInformativosController(ScreenService, templateManager, General) {

    this.backToLastWidget = function (widget) {
        let widgets = widget.container.widgets;
        let acwidget = widgets.indexOf(widget);

        widget.isVisible = false;

        let widgetToGo = widgets[acwidget - 1];

        widgetToGo.container.label = widgetToGo.servdata ? widgetToGo.servdata[widgetToGo.label] : null || widgetToGo.currentRow.servdata ? widgetToGo.currentRow.servdata[widgetToGo.label] : widgetToGo.label;

        widgetToGo.isVisible = true;
        widgetToGo.activate();
        if(isBackButtonListenerAttached){
            document.removeEventListener('backbutton', goBackListener);
            isBackButtonListenerAttached = false;
        }
    };

    var isBackButtonListenerAttached = false;

    let goBackListener = () => {
        const widget = templateManager.container.getWidget('openInformativo');
        this.backToLastWidget(widget);
    };

    this.bindBackButtonListener = () => {
        if('cordova' in window){
            isBackButtonListenerAttached = true;
            document.addEventListener('backbutton', goBackListener);
        }
    };

    this.getInformStatus = (container) => {
        let lastAccs = getLocalVar('lastAccessOnApp') || null;
        container.label =  getLocalVar('DSNOMEEXIBICAO') || ScreenService.i18n('Início');
        General.callRestEngine("/lastAccessOnApp", "Row", {
            'lastAccs': moment(lastAccs, 'DD/MM/YYYY HH:mm:ss').format('YYYYMMDDHHmmss')
        }).then(function (response) {
            let unread = response.dataset.count.count;
            let inforFooter = document.getElementById('rui-footer-element-informativos');
            if (unread > 0 && inforFooter) {
                inforFooter.classList.add('dot-of-new');
            }
        });
        let userData = getLocalVar('userData');
        container.getWidget('mainWidget').getAction('changePasswd').isVisible = !(userData && (userData.isAnon === true || userData.isFacebookConnected === true));

        
        if('FirebasePlugin' in window){
            window.FirebasePlugin.confirmListenerAttached();
        }
    };
    this.clearInformStatus = (container) => {
        let lastAccs = moment().format('DD/MM/YYYY HH:mm:ss');
        setLocalVar('lastAccessOnApp', lastAccs);
        let inforFooter = document.getElementById('rui-footer-element-informativos');
        if (inforFooter) {
            inforFooter.classList.remove('dot-of-new');
        }
    };

    this.openPendingDetails = () => {
        if(window.__pendingDetails){
            const informativoWidget = templateManager.container.getWidget("mainWidget");
            let informativoDetails = informativoWidget.dataSource.data.find(e => e.NRINFORMATIVO === window.__NRINFORMATIVO);
            if(informativoDetails){
                informativoWidget.goToDetails(informativoDetails, informativoWidget);
            }
            window.__pendingDetails = false;
        }
    };

    this.openInformativoFromPushNotification = (event) => {
        let details = event.detail;
        if(details.NRINFORMATIVO){
            if(templateManager.container.name !== "informativos") {
                openWindow("informativos");
                window.__pendingDetails = true;    
                window.__NRINFORMATIVO = details.NRINFORMATIVO;    
            } else {
                const informativoWidget = templateManager.container.getWidget("mainWidget");
                let informativoDetails = informativoWidget.dataSource.data.find(e => e.NRINFORMATIVO === details.NRINFORMATIVO);
                if(informativoDetails){
                    informativoWidget.goToDetails(informativoDetails, informativoWidget);
                }
            }
        } else {
            if (details.title) {
                ScreenService.showMessage(details.title + "<br>" + details.body);
            }
        }
    };

    document.addEventListener('notificationopen', this.openInformativoFromPushNotification);

}

Configuration(function (ContextRegister) {
    ContextRegister.register('MyMenuInformativosController', MyMenuInformativosController);
});

// FILE: ./assets/js/controllers/PesquisaController.js
function PesquisaController(ScreenService, General, templateManager, metaDataFactory, MessageService) {
    var pesquisaContainerName;
    var pesquisaContainer;
    var pesquisasWidget;
    var currentPesq;

    this.openItemPesquisa = function () {
        ScreenService.openWindow("product#item_pesquisa");
    };
    this.closeItemPesquisa = function () {
        ScreenService.openWindow("product#pesquisa");
    };

    this.itemPesquisaOnOpen = function (itemPesquisaWidget) {
        var containers = templateManager.containers;
        var historyItems = templateManager.historyItems;


        if (containers && historyItems)
            pesquisaContainerName = historyItems[historyItems.length - 2];
        pesquisaContainer = containers[pesquisaContainerName];

        if (pesquisaContainer) {
            pesquisasWidget = pesquisaContainer.getActiveWidget();
            currentPesq = pesquisasWidget.currentRow;
            itemPesquisaWidget.container.label = currentPesq.NMQUESTIONARIO;
            itemPesquisaWidget.label = currentPesq.NMQUESTIONARIO;

            clearFields(itemPesquisaWidget);

            var filter = [{
                "name": "NRQUESTIONARIO",
                "operator": "=",
                "value": currentPesq.NRQUESTIONARIO
            }];
            itemPesquisaWidget.reload(filter).then(function () {
                buildFields(itemPesquisaWidget);
                itemPesquisaWidget.newRow();
                itemPesquisaWidget.currentRow.NRPROGRAMACAOPESQ = currentPesq.NRPROGRAMACAOPESQ;
                itemPesquisaWidget.currentRow.NRQUESTIONARIO = currentPesq.NRQUESTIONARIO;
            });

        }
    };

    function clearFields(itemPesquisaWidget) {
        itemPesquisaWidget.fields = [];
    }

    function buildFields(itemPesquisaWidget) {
        var templates = {
            ABERTA: 'field/textarea.html',
            FECHADA_UNI: 'field/select-native.html',
            FECHADA_MUL: 'mymenu#fields/select-autocomplete-multiple.html'
        };

        var fieldNames = [];
        itemPesquisaWidget.dataSource.data.forEach(function (itemQuestionario) {
            var fieldName = 'QUESTAO' + itemQuestionario.NRQUESTAO;

            if (fieldNames.indexOf(fieldName) === -1) {
                let canHaveNextQuestion = itemQuestionario.IDTIPOQUESTAO !== 'FECHADA_MUL';
                fieldNames.push(
                    buildField(
                        itemPesquisaWidget, 
                        itemQuestionario, 
                        templates[itemQuestionario.IDTIPOQUESTAO], 
                        true, 
                        undefined, 
                        canHaveNextQuestion
                ));
            } else {
                var field = itemPesquisaWidget.getField(fieldName);
                field.dataSource.data.push(buildResp(itemQuestionario));
            }
        });

        buildSubmmitBtn(itemPesquisaWidget);
    }

    /**
     * @param {*} itemPesquisaWidget 
     * @param {*} itemQuestionario 
     * @param {*} template 
     * @param {boolean} [shouldAppendToWidget] 
     * @param {string} newFieldName 
     * @param {boolean} [canHaveNextQuestion]
     * @returns {Field|string}
     */
    function buildField(itemPesquisaWidget, 
                            itemQuestionario, 
                            template, 
                            shouldAppendToWidget = true, 
                            newFieldName = undefined, 
                            canHaveNextQuestion = true) {
        var fieldName = newFieldName;
        if (!fieldName) {
            fieldName = 'QUESTAO' + itemQuestionario.NRQUESTAO;
        }
        
        var field = {
            name: fieldName,
            label: itemQuestionario.TXQUESTAO,
            isVisible: true,
            template: template,
            class: 12,
            fields: [{
                    name: "value",
                    isVisible: false,
                    showOnList: false
                },
                {
                    name: "nextQuestionID",
                    isVisible: false,
                    showOnList: false
                },
                {
                    name: "desc",
                    // label: "Opções",
                    isVisible: true,
                    showOnList: true
                }
            ],
            valueField: "value",
            descriptionField: "desc",
            // outData: {},
            preventAutoFocus: true,
            dataSource: {
                data: [
                    buildResp(itemQuestionario)
                ]
            },
            descriptionFields: ["desc"],
        };

        if(itemQuestionario.IDQUESTAOOBRIGA === 'S'){
            field.validations = {
                required:  true
            };
            field.hideEmptyValue = true;
        }

        if(canHaveNextQuestion){
            field.events = [{
                "name": "FieldOnChange",
                "code": "PesquisaController.setDescriptionOnCurrentRow(args);PesquisaController.loadDependentQuestion(args)"
            }];
        } else {
            field.events = [{
                "name": "FieldOnChange",
                "code": "PesquisaController.setDescriptionOnCurrentRow(args);"
            }];
        }

        if(typeof template === 'string' && !template.includes('textarea')){
            field.outData = {};
            field.outData["description|" + fieldName] = 'desc';
        }
        // field.outData[outDataPropertyName] = "NRPROXIMAQUESTAO";
        // field.outData.NRPROXIMAQUESTAO = outDataPropertyName;
        let factoredField = metaDataFactory.fieldFactory(field, itemPesquisaWidget, shouldAppendToWidget);
        if (shouldAppendToWidget) {
            return fieldName;
        } else {
            return factoredField;
        }
    }

    let setDescriptionSelectNative = (field) => {
        if(field.value()){
            let answerDescription = field.modelValue();
            field.widget.currentRow['description|' + field.name] = answerDescription;
        } else {
            delete field.widget.currentRow['description|' + field.name];
        }
    };

    // all description setters must be arrow functions in order for the scope to work correctly
    let descriptionSetters = {
        "field/select-native.html": setDescriptionSelectNative,
    };

    this.setDescriptionOnCurrentRow = (args) => {
        let field = args.owner.field;
        if(descriptionSetters.hasOwnProperty(field.template)){
            descriptionSetters[field.template].call(null, field);
        }
    };

    

    this.buildDependent = (dataSet, parentFieldName, widget) => {
        let pesquisasWidget = widget;
        let itemQuestionario = dataSet.dataset.qst_especifica;

        var templates = {
            ABERTA: 'field/textarea.html',
            ABERTA_TEXTO: 'field/textarea.html',
            FECHADA: 'field/select-native.html',
            FECHADA_UNI: 'field/select-native.html',
            FECHADA_MUL: 'mymenu#fields/select-autocomplete-multiple.html'
        };

        const IDTIPOQUESTAO = itemQuestionario.IDTIPOQUESTAO;
        const IDOPCAORESPOSTA = itemQuestionario.IDOPCAORESPOSTA;
        let questionType = IDTIPOQUESTAO;
        questionType = (IDTIPOQUESTAO === "FECHADA" && IDOPCAORESPOSTA === "MULTIPLAS" && "FECHADA_MUL") || questionType; 

        let canHaveNextQuestion = questionType !== 'FECHADA_MUL';
        let newField = buildField(pesquisasWidget, itemQuestionario, templates[questionType],
            false, 'QUESTAO' + itemQuestionario.NRQUESTAO + "_" + parentFieldName, canHaveNextQuestion);
        let parentFieldIndex = pesquisasWidget.fields.findIndex(field => field.name === parentFieldName);
        widget.fields.splice(parentFieldIndex + 1, 0, newField);
        newField.parentFieldName = parentFieldName;


        newField.dataSource.data = dataSet.dataset.resp_especifica.map(itemQuestionario => buildResp(itemQuestionario));
        // dataSet.dataset.resp_especifica.forEach(itemQuestionario => {
        //     // var fieldName = 'QUESTAO'+itemQuestionario.NRQUESTAO;    
        //     newField.dataSource.data.push(buildResp(itemQuestionario));
        // });

        // buildField(pesquisasWidget, );
    };

    this.removeDependentQuestion = (questionID, parentName, widget) => {
        // widget.fields.()
        let dependentField = widget.fields.find(field => field.name === ('QUESTAO' + questionID + '_' + parentName) &&
            field.parentFieldName === parentName);
        if (dependentField && dependentField.dependentID) {
            this.removeDependentQuestion(dependentField.dependentID, dependentField.name, widget);
            delete dependentField.dependentID;
            delete widget.currentRow[('QUESTAO' + questionID + '_' + parentName)];
        }
        widget.fields = widget.fields.filter(field => field !== dependentField); //.splice(, 1);
        delete widget.currentRow[dependentField.name];
    };

    let isEmpty = (valueToCheck) => valueToCheck === "null" || valueToCheck === null ||
        valueToCheck === 'undefined' || valueToCheck === undefined;

    this.loadDependentQuestion = function (args) {
        let fieldName = args.owner.field.name;
        let nextQuestionID;
        
        if (args.row[fieldName]) {
            let splited = args.row[fieldName].split("@@");
            nextQuestionID = splited[1];
        } else {
            nextQuestionID = null;
        }
        // args.row[fieldName] = splited[0];
        if (!isEmpty(nextQuestionID) && !args.owner.field.dependentID) {
            args.owner.field.dependentID = nextQuestionID;
            General.callRestEngine("/getNextQuestion", "Row", {
                qstid: nextQuestionID
            }).then((response) => this.buildDependent(response, fieldName, args.owner.field.widget));
        } else if (args.owner.field.dependentID && nextQuestionID !== args.owner.field.dependentID) {
            this.removeDependentQuestion(args.owner.field.dependentID, args.owner.field.name, args.owner.field.widget);
            delete args.owner.field.dependentID;
            if (!isEmpty(nextQuestionID)) {
                args.owner.field.dependentID = nextQuestionID;
                General.callRestEngine("/getNextQuestion", "Row", {
                    qstid: nextQuestionID
                }).then((response) => this.buildDependent(response, fieldName, args.owner.field.widget));
            }
        }
        
    };

    function buildResp(itemQuestionario) {
        return {
            value: itemQuestionario.NRRESPOSTA + "@@" + itemQuestionario.NRPROXIMAQUESTAO,
            desc: itemQuestionario.DSRESPOSTA,
        };
    }

    function buildSubmmitBtn(itemPesquisaWidget) {
        var field = {
            name: "submmit",
            label: "Enviar",
            isVisible: true,
            template: "field/button.html",
            class: 12,
            events: [{
                name: "FieldOnClick",
                code: "PesquisaController.pesquisaSubmmit(args.owner.owner.widget);"
            }]
        };
        metaDataFactory.fieldFactory(field, itemPesquisaWidget, true);
    }

    function stripATchar(row) {
        let stripedRow = {};
        for (const key in row) {
            if (row[key] && typeof row[key] === 'string') {
                stripedRow[key] = row[key].split("@@")[0];
            } else if(row[key] && Array.isArray(row[key])){
                stripedRow[key] = row[key].map(answer => answer.split("@@")[0]);    
            }
        }
        return stripedRow;
    }

    let textareaRowBuilder = (field, newRow) => {
        const fieldName = field.name;
        const currentRow = field.widget.currentRow;
        if(currentRow[fieldName]){
            const parentAnswer = currentRow[field.parentFieldName] && currentRow[field.parentFieldName].split("@@")[0];
            newRow[fieldName] = {
                value: null,
                description: currentRow[fieldName],
                parentAnswer: parentAnswer || null
            };
        }
    };
    
    let selectNativeRowBuilder = (field, newRow) => {
        const fieldName = field.name;
        const currentRow = field.widget.currentRow;
        if(currentRow[fieldName]){
            const parentAnswer = currentRow[field.parentFieldName] && currentRow[field.parentFieldName].split("@@")[0];
            newRow[fieldName] = {
                value: currentRow[fieldName].split("@@")[0],
                description: currentRow["description|" + fieldName],
                parentAnswer: parentAnswer || null
            };
        }
    };

    let selectAutocompleteMultipleRowBuilder = (field, newRow) => {
        const fieldName = field.name;
        const currentRow = field.widget.currentRow;
        if(currentRow[fieldName] && currentRow[fieldName].length > 0){
            const descriptionKey = 'description|' + fieldName;
            const parentAnswer = currentRow[field.parentFieldName] && currentRow[field.parentFieldName].split("@@")[0];
            newRow[fieldName] = currentRow[fieldName].map((selectedElement) => {
                return {
                    value: selectedElement.value.split("@@")[0],
                    description: selectedElement[descriptionKey],
                    parentAnswer: parentAnswer || null
                };
            });
        }
    };

    let rowBuilders = {
        "field/textarea.html": textareaRowBuilder,
        "field/select-native.html" : selectNativeRowBuilder, 
        "mymenu#fields/select-autocomplete-multiple.html" : selectAutocompleteMultipleRowBuilder, 
    };

    function buildRowV2(widget) {
        let newRow = {};
        widget.fields.forEach((field) => {
            if(rowBuilders.hasOwnProperty(field.template)){
                rowBuilders[field.template].call(null, field, newRow);
            }
        });
        newRow.surveyData = {
            NRPROGRAMACAOPESQ : widget.currentRow.NRPROGRAMACAOPESQ,
            NRQUESTIONARIO : widget.currentRow.NRQUESTIONARIO
        };
        return newRow;
    }
    
    
    function buildRow(widget) {
        let row = stripATchar(widget.currentRow);
        for (const key in row) {
            let field = widget.getField(key);
            if (field && field.template.includes("textarea")) {
                row[key] = {
                    value: row[key],
                    type: "open"
                };
            }
        }
        return row;
    }

    this.filterForAnon = function (widget) {
        if (getLocalVar('userData').isAnon && getLocalVar('userData').isAnon === true && getLocalVar('lastQuestAsw')) {
            let list = getLocalVar('lastQuestAsw');
            let toBeRemoved = [];
            list.forEach((it,key)=>{
                if(moment().format('DD/MM/YYYY') === it.date){
                    let toRemove = widget.dataSource.data.find(d=>d.NRPROGRAMACAOPESQ === it.NRPROGRAMACAOPESQ && d.NRQUESTIONARIO === it.NRQUESTIONARIO);
                    if(toRemove){
                        widget.dataSource.data.splice(widget.dataSource.data.indexOf(toRemove),1);
                    }    
                }else{
                    toBeRemoved.push(key);
                }
            });
            toBeRemoved.forEach(tb =>{
                list.splice(tb,1);
            });
            setLocalVar('lastQuestAsw',list);
        }
    };
    this.pesquisaSubmmit = function (itemPesquisaWidget) {
        if (itemPesquisaWidget.isValid()) {
            if (getLocalVar('userData').isAnon && getLocalVar('userData').isAnon === true) {
                let last = {
                    "date": moment().format('DD/MM/YYYY'),
                    "NRQUESTIONARIO": itemPesquisaWidget.currentRow.NRQUESTIONARIO,
                    "NRPROGRAMACAOPESQ": itemPesquisaWidget.currentRow.NRPROGRAMACAOPESQ
                };
                if(getLocalVar('lastQuestAsw')){
                    let arr = getLocalVar('lastQuestAsw');
                    arr.push(last);
                    setLocalVar('lastQuestAsw', arr);
                }else{
                    setLocalVar('lastQuestAsw', [last]);
                }
            }

            General.callRestEngine('/pesquisaSubmmit', 'Row', buildRowV2(itemPesquisaWidget)).then(
                function () {
                    MessageService.showMessageByCode('MSG_SENT_RATING_WITH_SUCCESS', null, {type:'success'});
                    ScreenService.openWindow(pesquisaContainerName);
                    pesquisasWidget.reload();
                    currentPesq = {};
                }
            );
        }
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register('PesquisaController', PesquisaController);
});

// FILE: ./assets/js/controllers/QuestionaryController.js
function QuestionaryController(ScreenService, General, metaDataFactory) {
    this.onEnter = function (questionaryWidget) {
        if (questionaryWidget.container.questionary) {
            if (questionaryWidget.container.questionary.twiceCallCorrection) {
                return false; // correcao do bug de chamada duplicada do onEnter
            } else {
                questionaryWidget.container.questionary.twiceCallCorrection = true;
            }

            questionaryWidget.fields = [];

            var editWidget = function(widget){
                widget.edit();
                widget.template = widget.templateToForm;
            };
            var viewWidget = function (widget) {
                widget.view();
                widget.template = widget.templateToView;
            };

            var modeDecode = {
                "Disponível": editWidget,
                "Em andamento": editWidget,
                "Finalizado": viewWidget
            };

            questionaryWidget.currentRow = questionaryWidget.container.questionary;
            var currentRow = questionaryWidget.currentRow;
            
            var modeFunc = modeDecode[currentRow.STATUS];
            modeFunc(questionaryWidget);
            
            questionaryWidget.container.label = currentRow.NMQUESTIONARIO;

            var filter = [
                {
                    "name": "NRQUESTIONARIO",
                    "operator": "=",
                    "value": currentRow.NRQUESTIONARIO
                },
                {
                    "name": "NRPROGRAMACAOPESQ",
                    "operator": "=",
                    "value": currentRow.NRPROGRAMACAOPESQ
                },
                {
                    "name": "NREXECUCAOPESQ",
                    "operator": "=",
                    "value": currentRow.NREXECUCAOPESQ
                }
            ];
            General.callRestEngine('/getQuestionaryExec', 'FilterData', filter).then(function(response){
                if (response && response.dataset) {
                    var questoes = response.dataset.questionario_questoes || [];
                    var respostas = response.dataset.questionario_respostas_execucao || [];

                    questionaryWidget.fields = [];
                    questionaryWidget.fieldSequence = 0;
                    questoes.forEach(function(questao) {
                        var fieldName = 'NRQUESTAO' + questao.NRQUESTAO;                        
                        var newField = {
                            "isVisible": true,
                            "name": fieldName,
                            "label": questao.TXQUESTAO,
                            "template": getFieldTemplate(questao),
                            "class": 12,
                            "validations": getFieldValidations(questao),
                            "treatAnswer": getFunctionForTreatAnswer(questao),
                            "dataSource": {
                                "data": []
                            },
                            "valueField": "NRRESPOSTA",
                            "descriptionField": "DSRESPOSTA"
                        };
                        metaDataFactory.fieldFactory(newField, questionaryWidget, true);
                    });
                    if (questionaryWidget.currentRow.STATUS != "Finalizado") {
                        var finalizarField = {
                            "name": "finalizar",
                            "label": "Finalizar",
                            "isVisible": true,
                            "template": "field/button.html",
                            "class": 12
                        };
                        metaDataFactory.fieldFactory(finalizarField, questionaryWidget, true);
                    }

                    respostas.forEach(function(resposta) {
                        var fieldName = 'NRQUESTAO' + resposta.NRQUESTAO;
                        var questaoField = questionaryWidget.getField(fieldName);
                        questaoField.treatAnswer(questaoField, resposta);
                    });
                }
            });
        } else {
            ScreenService.openWindow("home");
        }
    };

    var getFieldValidations = function (questao) {
        return (questao.IDQUESTAOOBRIGA == 'S') ? { "required": true } : null;
    };
    var getFieldTemplate = function (questao) {
        var idtipoquestao = questao.IDTIPOQUESTAO;
        var idopcaoresposta = questao.IDOPCAORESPOSTA;

        if (idtipoquestao === "ABERTA_TEXTO") {
            return "field/text-edit.html";
        } else if (idtipoquestao === "FECHADA") {
            if (idopcaoresposta === "UNICA") {
                return "field/select-native.html";
            } else if (idopcaoresposta === "MULTIPLAS") {
                return "field/select-multiple.html";                
            }
        }
        return "field/text-edit.html";
    };
    var getFunctionForTreatAnswer = function (questao) {
        var idtipoquestao = questao.IDTIPOQUESTAO;
        var idopcaoresposta = questao.IDOPCAORESPOSTA;

        if (idtipoquestao === "ABERTA_TEXTO") {
            return function (questaoField, resposta) {
                questaoField.setValue(resposta.DSRESPOSTA);
            };
        } else if (idtipoquestao === "FECHADA") {
            if (idopcaoresposta === "UNICA") {
                return function (questaoField, resposta) {
                    if (resposta.IDRESPOSTA === "S") {
                        questaoField.setValue(resposta.NRRESPOSTA);
                    }
                    questaoField.dataSource.data.push(resposta);
                };
            } else if (idopcaoresposta === "MULTIPLAS") {
                return function (questaoField, resposta) {
                    if (resposta.IDRESPOSTA === "S") {
                        var currentRow = questaoField.widget.currentRow;
                        var fieldName = questaoField.name;
                        if (Array.isArray(currentRow[fieldName])) {
                            currentRow[fieldName].push(resposta.NRRESPOSTA);
                        } else {
                            currentRow[fieldName] = [resposta.NRRESPOSTA];
                        }
                        resposta.checked = true;
                    }
                    questaoField.dataSource.data.push(resposta);
                };
            }
        }
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register('QuestionaryController', QuestionaryController);
});

// FILE: ./assets/js/controllers/RecuperarSenhaController.js
function RecuperarSenhaController(ScreenService, General, MessageService) {
    
    var self = this;
    var currentStep;
    
    this.onEnter = function(widget){
        toFirstStep(widget);
        widget.newRow();
    };
    
    this.submit = function(widget) {
        if (widget.isValid()) {
            if (currentStep == 1) {
                General.callRestEngine("/sendPwRecoverToken", "Row", widget.currentRow).then(function(){
                    MessageService.showMessageByCode('MSG_SENT_TOKEN');
                    self.toSecondStep(widget);
                }, function(response){
                    var errorMsg = "Erro";
                    errorMsg = response.error || (response.data && response.data.error) || errorMsg;
                    ScreenService.showMessage(ScreenService.i18n(errorMsg), 'E');
                });
            } else if (currentStep == 2) {
                var currentRow = widget.currentRow;
                if (currentRow.newPw == currentRow.confirmPw) {
                    General.callRestEngine("/resetPassword", "Row", widget.currentRow).then(function(){
                        MessageService.showMessageByCode('MSG_SUCCESS_RECOVER');
                        ScreenService.openWindow('product#login');
                    }, function(response){
                        var errorMsg = "Erro";
                        errorMsg = response.error || (response.data && response.data.error) || errorMsg;
                        ScreenService.showMessage(ScreenService.i18n(errorMsg), 'E');
                    });
                } else {
                    MessageService.showMessageByCode('MSG_VALID_PWRECOVER');
                    currentRow.newPw = "";
                    currentRow.confirmPw = "";
                }
            }
            
        }
    };
    
    this.cancel = function(widget){
        ScreenService.openWindow('product#login');
    };
    
    var toFirstStep = function(widget){
        currentStep = 1;
        widget.getField('token').isVisible = false;
        widget.getField('newPw').isVisible = false;
        widget.getField('confirmPw').isVisible = false;
        widget.getField('toSecondStep').isVisible = true;
    };
    
    this.toSecondStep = function(widget){
        currentStep = 2;
        widget.getField('token').isVisible = true;
        widget.getField('newPw').isVisible = true;
        widget.getField('confirmPw').isVisible = true;
        widget.getField('toSecondStep').isVisible = false;
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register('RecuperarSenhaController', RecuperarSenhaController);
});

// FILE: ./assets/js/util/FCMBootstrap.js
let FCMBootstrap = function() {

    /**
     * @param {string} topic 
     */
    let unsubscribe = (topic) => {
        return new Promise((resolve, reject) => {
            if('FirebasePlugin' in window){
                FirebasePlugin.unsubscribe(topic, (s) => {
                    localStorage.removeItem("subscribedTopic");
                    resolve(s);
                }, (e) => reject(e));
            } else {
                reject();
            }
        });
    };

    /**
     * @param {string} topic 
     */
    let subscribe = (topic) => {
        return new Promise((resolve, reject) => {
            if('FirebasePlugin' in window){
                FirebasePlugin.subscribe(topic, (s) =>{ 
                    localStorage.setItem("subscribedTopic", topic);
                    resolve(s);
                }, (e) => reject(e));
            } else {
                reject();
            }
        });
    };

    this.start = () => {
        let qrCodeData = getLocalVar('qrCodeData');
        let customerID = getLocalVar('customerID');

        if(qrCodeData && customerID){
            let subscribedTopic = localStorage.getItem("subscribedTopic");
            let newTopicName = customerID + "_" + qrCodeData.nrorg + "_" + qrCodeData.nrqrcode;
            if(subscribedTopic){
                return unsubscribe(subscribedTopic).then(() => subscribe(newTopicName));
            } else {
                return subscribe(newTopicName);
            }
        } else {
            return Promise.reject();
        }

    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register('FCMBootstrap', FCMBootstrap);
});


// FILE: ./assets/js/util/UIManager.js
let UIManager = function(eventAggregator, templateManager) {

    /**
     * @type {Array<String|Function>}
     */
    let stateStack = [];

    let isListenerAttached = false;
    let activeWidget;
    let sf = this;
    /**
     * Called every time a container is open
     */
    this.init = () => {
        stateStack = [];
        activeWidget = this.activeWidget = templateManager.container.getActiveWidget();
    };

    if (!isListenerAttached) {
        eventAggregator.subscribe('onContainerOpen', this.init);
        isListenerAttached = true;
    }

    /**
     * Displays the given widget
     * @param {ZhWidgetOjbect} widgetObject
     * @param {boolean} [shouldReload] 
     */
    this.goToWidget = (widgetObject, shouldReload = false) => {
        if (widgetObject && widgetObject.name !== activeWidget.name) {
            widgetObject.activate();
            stateStack.push(activeWidget.name);
            activeWidget.isVisible = false;
            widgetObject.isVisible = true;
            activeWidget = widgetObject;
            if (activeWidget.container.steps) {
                templateManager.container.highlightStep(widgetToOpen.name);
                sf.moveSteps(widgetToOpen);
            }
            if (shouldReload) {
                return widgetObject.reload();
            }
        }
    };

    /**
     * Tryes to find a widget by it's name and display it
     * @param {string} widgetName 
     * @param {boolean} shouldReload 
     */
    this.goToWidgetByName = (widgetName, shouldReload = false, shouldCarryCurRow = false) => {
        let widgetToOpen = templateManager.container.getWidget(widgetName);
        if (widgetToOpen && widgetToOpen.name !== activeWidget.name) {
            widgetToOpen.activate();
            stateStack.push(activeWidget.name);
            activeWidget.isVisible = false;
            widgetToOpen.isVisible = true;
            if (shouldCarryCurRow) {
                widgetToOpen.currentRow = activeWidget.currentRow;
            }
            activeWidget = widgetToOpen;
            if (activeWidget.container.steps) {
                templateManager.container.highlightStep(widgetToOpen.name);
                sf.moveSteps(widgetToOpen);
            }

            if (shouldReload) {
                return widgetToOpen.reload();
            }
        } else {
            console.log(`%can attempt to open the widget: ${widgetName} was made. But this widget wasn't found`, "font-size: 15px");
        }
    };

    /**
     * Displays a widget that is already in the stack 
     * and removes all others in the path
     * 
     * homeScreen -> list -> listItem -> listItemDetails
     * 
     * goBackToStackedWidget('list')
     * 
     * homeScreen
     *  
     * @param {string} widgetName 
     */
    this.goBackToStackedWidget = widgetName => {
        let widgetToGoBack = templateManager.container.getWidget(widgetName);
        let widgetNameIndex = stateStack.indexOf(widgetName);
        if (widgetToGoBack && widgetNameIndex !== -1 && stateStack.length > 0) {
            stateStack = stateStack.slice(0, widgetNameIndex);
            widgetToGoBack.activate();
            activeWidget.isVisible = false;
            widgetToGoBack.isVisible = true;
            activeWidget = widgetToGoBack;
        }
    };

    /**
     * Displays the last widget in the stack
     * or triggers the last callback state in the stack 
     */
    this.goBack = () => {
        if (stateStack.length > 0) {
            if (typeof stateStack[stateStack.length - 1] === 'string') {
                let widgetToGoBackName = stateStack.pop();
                let widgetToGoBack = templateManager.container.getWidget(widgetToGoBackName);
                widgetToGoBack.activate();
                widgetToGoBack.isVisible = true;
                activeWidget.isVisible = false;

                let steps = widgetToGoBack.container.steps;
                if (steps && steps.length > 0) {
                    let footerPositioner = document.getElementsByClassName("footer-positioner")[0];
                    let stepElements = document.getElementsByClassName("footer-wizard-step");

                    steps.find(step => step.active === true).active = false;
                    let newActiveStep = steps.find(step => step.name === widgetToGoBackName);
                    let indexOfStep = steps.indexOf(newActiveStep);
                    newActiveStep.active = true;

                    let curStep = stepElements[indexOfStep];
                    let curStepWidth = parseInt(window.getComputedStyle(curStep, null).getPropertyValue("width"));
                    footerPositioner.scrollLeft = curStep.offsetLeft - (curStepWidth / 3);
                }
                activeWidget = widgetToGoBack;
            } else if (typeof stateStack[stateStack.length - 1] === 'function') {
                (stateStack.pop())();
            }
        }
    };

    /**
     * hides the editor of the active widget, (if this widget is editable)
     */
    this.hideEdit = () => {
        if (activeWidget.hasOwnProperty('isEditing')) {
            this.allowScroll();
            activeWidget.isEditing = false;
            // activeWidget.restoreHeaderActions();
        }
    };

    this.preventScroll = () => document.body.classList.add('no-scroll');

    this.allowScroll = () => document.body.classList.remove('no-scroll');

    this.moveSteps = (widget) => {
        let steps = widget.container.steps;
        if (steps) {
            let footerPositioner = document.getElementsByClassName("footer-positioner")[0];
            let stepElements = document.getElementsByClassName("footer-wizard-step");

            let curStep = stepElements[steps.indexOf(steps.find(step => step.active === true))];
            let curStepWidth = parseInt(window.getComputedStyle(curStep, null).getPropertyValue("width"));
            footerPositioner.scrollLeft = curStep.offsetLeft - (curStepWidth / 3);
        }
    };

    /**
     * @param {Function} callback
     */
    this.pushState = (callback) => stateStack.push(callback);

};

Configuration(function(ContextRegister) {
    ContextRegister.register('UIManager', UIManager);
});


// FILE: ./assets/templates/controllers/CardapioController.js
function CardapioController(ApplicationContext, ScreenService, MessageService) {
    let sf = this;
    const {LocalPlateDataService, templateManager} = ApplicationContext;

    const getLimitDate = (limitHours, limitDays) => {
        const [hours, minutes] = limitHours.split(':');
        const limitTime = moment();
        const days = parseInt(limitDays);
        limitTime.add({minutes, hours, days});
        return limitTime;    
    };

    this.getConfigSize = () => {
        if (getLocalVar('IDTAMCARDIMG')) {
            if (getLocalVar('IDTAMCARDIMG') === 'S') {
                return 'md';
            } else if (getLocalVar('IDTAMCARDIMG') === 'H') {
                return 'sm';
            }
        }
    };

    this.getTitleFiledComposition = (servdata, field) => {
        return servdata[field.code] ? servdata[field.code] + ' | ' + servdata[field.name] : servdata[field.name];
    };

    this.getIDEXIBENUTR = () => {
        return (getLocalVar('IDEXIBENUTR') && getLocalVar('IDEXIBENUTR') === 'N') ? false : true;
    };

    this.getTimeFromDataSourceFilter = (widget) => {
        widget.dataSourceFilter.find(f => f.name === widget.dateName);

        let curDate = widget.dataSourceFilter.find(f => f.name === 'DTCARDAPIO');
        curDate = moment(curDate.value, 'DD/MM/YYYY').format('YYYY-MM-DD') === "Invalid date" ? moment().format('YYYY-MM-DD') : moment(curDate.value, 'DD/MM/YYYY').format('YYYY-MM-DD');
        widget.dateFiltred = curDate;

        return moment(curDate, 'DD/MM/YYYY').format('DD/MM/YYYY') === "Invalid date" ? moment(curDate, 'YYYY-MM-DD').format('ddd - DD/MM') : moment(curDate.value, 'DD/MM/YYYY').format('ddd - DD/MM');
    };

    this.changeTimeFromDataSourceFilter = (widget) => {
        if (widget.dateFiltred > widget.DTMAX) {
            widget.dateFiltred = widget.DTMAX;
        } else if (widget.dateFiltred < widget.DTMIN) {
            widget.dateFiltred = widget.DTMIN;
        }
        widget.dataSourceFilter.find(f => f.name === 'DTCARDAPIO').value = moment(widget.dateFiltred, 'YYYY-MM-DD').format('DD/MM/YYYY');
        sf.getTimeFromDataSourceFilter(widget);
        widget.reload();
    };

    this.moveOneDay = (direction, widget) => {
        let actualDate = widget.dataSourceFilter.find(f => f.name === 'DTCARDAPIO').value;
        actualDate = actualDate === "@moment().format('DD/MM/YYYY')@" ? moment().format('DD/MM/YYYY') : actualDate;

        let newDate;
        if (direction) {
            //foward
            newDate = moment(actualDate, 'DD/MM/YYYY').add(1, 'd');
        } else {
            newDate = moment(actualDate, 'DD/MM/YYYY').subtract(1, 'd');
        }
        widget.dataSourceFilter.find(f => f.name === 'DTCARDAPIO').value = newDate.format('DD/MM/YYYY');
        sf.getTimeFromDataSourceFilter(widget);
        widget.reload();
    };

    this.showTime = (time) => {
        return time === ': - :' ? '' : time;
    };

    this.showLimitLabel = (serv) => {
        let status = (serv.HRINICIO === ':') ? 'Adiantamento de: ' : 'Antecedência de: ';
        return serv.HRLIMSEGPEDIDO !== null ? status + serv.HRLIMSEGPEDIDO.substring(0, 2) + ':' + serv.HRLIMSEGPEDIDO.substring(2, 4) : '';
    };

    this.hasImage = (objToFind, listConfig) => {
        return (listConfig.imageURL && (objToFind.IMGTHUMB || objToFind[listConfig.imageURL])) || objToFind[listConfig.bs4IMG];
    };

    this.getBackImg = (objToFind, listConfig) => {
        let url = getLocalVar("qrCodeURL");
        let defaultImgUrl = "assets/images/icons/no-picture-camera-100.png";
        if (listConfig.imageURL && !listConfig.bs4IMG) {
            // let thumbImg = objToFind.IMGTHUMB;
            let thumbImg = objToFind.IMGSERVICO;
            let imgPath = thumbImg ? thumbImg : objToFind[listConfig.imageURL];
            return imgPath ? imgPath : defaultImgUrl;
        } else {
            let imgPath = objToFind[listConfig.bs4IMG];
            return imgPath ? 'data:image/png;base64,' + imgPath : defaultImgUrl;
        }
    };
    
    
    this.getPlateImage = (plate, listConfig) => {
        let baseUrl =  window.__backendURL || (window.__backendURL = getLocalVar("backendURL"));
        const {NRFOTOPRATOBLOB} = plate;
        const url = `${baseUrl}/plateimage?NRFOTOPRATOBLOB=${NRFOTOPRATOBLOB}`;
        return NRFOTOPRATOBLOB ? {
            "background-image": `url(${url})`
        } : {};
    };
    
    this.goToDetails = (servdata, widget) => {
        let widgets = widget.container.widgets;
        let acwidget = widgets.indexOf(widget);

        if (widgets[acwidget + 1]) {
            let widgetToGo = widgets[acwidget + 1];

            let IDEXIBENUTR = getLocalVar('IDEXIBENUTR') ? getLocalVar('IDEXIBENUTR') : 'S';
            if (widgetToGo.name === "nutrients" && IDEXIBENUTR === 'N') {
                return false; //Should not show nutrients screen when not available (IDEXIBENUTR === 'N')
            }

            widget.isVisible = false;
            widgetToGo.container.label = servdata[widgetToGo.label];
            let filterToInherit = widgetToGo.filterToInherit;
            widgetToGo.dataSourceFilter = [];

            if (filterToInherit && widgetToGo.name !== "nutrients") {
                filterToInherit.forEach(fil => {
                    if (fil === 'DTCARDAPIO') {
                        servdata[fil] = widgets[acwidget].dataSourceFilter.find(f => f.name === 'DTCARDAPIO').value;
                    }
                    widgetToGo.dataSourceFilter.push({
                        'name': fil,
                        'operator': '=',
                        'value': servdata[fil] || null
                    });
                });
            } else if (widgetToGo.name === "nutrients") {
                widgetToGo.filterToInherit.forEach(fil => {
                    widgetToGo.dataSourceFilter.push({
                        'name': fil,
                        'operator': '=',
                        'value': servdata[fil] || widget.currentRow.servdata[fil]
                    });
                });
                widgetToGo.dataSourceFilter.push({
                    'name': 'NRORG',
                    'operator': '=',
                    'value': getLocalVar('userData').NRORG
                });
            }
            if (widgetToGo.name === "plates") {
                widgetToGo.dataSourceFilter.push({
                    'name': 'NRUSUARIOAPP',
                    'operator': '=',
                    'value': getLocalVar('userData').NRUSUARIOAPP
                });
                widgetToGo.dataSourceFilter.push({
                    'name': 'NRLOCALPESQ',
                    'operator': '=',
                    'value': getLocalVar('NRLOCALPESQ')
                });
            }

            widgetToGo.isVisible = true;
            widgetToGo.currentRow.servdata = servdata;
            widgetToGo.activate();
            widgetToGo.reload();
        }
    };

    this.rateIt = (widget, star) => {
        let stars = widget.stars;
        stars.forEach(st => {
            if (st === star) {
                star.active = true;
            } else {
                st.active = false;
            }
        });
        widget.plateToRate.newAv = star.value;
    };

    this.doAction = (code, widget) => {
        if (widget.plateToRate && widget.plateToRate.newAv) {

            widget.stars.forEach(st => {
                st.active = false;
            });
            let $scope = angular.element(document.getElementsByTagName("body")).scope();
            $scope.$eval(code, Object.assign({}, ApplicationContext, {
                args: {
                    owner: $scope.widget
                }
            }));
        } else {
            MessageService.showMessageByCode('MSG_INCOMPLETE_RATING');
        }

    };

    this.cancelRate = (widget) => {
        widget.plateToRate = null;
        widget.stars.forEach(st => {
            st.active = false;
        });
    };

    this.setStarValue = (widget) => {
        let stars = widget.stars;
        stars.forEach(st => {
            st.active = false;
        });
        let vl = widget.currentRow.servdata.VRAVALIACAO;
        let svl = stars.find(st => st.value == Math.round(vl));
        if (svl) {
            svl.active = true;
        }
    };

    function addServDataToPlate(plate, widget) {
        plate.CDFILCENTPME = widget.currentRow.servdata.CDFILCENTPME;
        plate.CDFILIAL = widget.currentRow.servdata.CDFILIAL;
        plate.CDSERVFILI = widget.currentRow.servdata.CDSERVFILI;
        plate.CDDIETPAD = widget.currentRow.servdata.CDDIETPAD;
        plate.CDTIPOCARD = widget.currentRow.servdata.CDTIPOCARD;
        plate.DTCARDAPIO = widget.currentRow.servdata.DTCARDAPIO;
        plate.NRORG = widget.currentRow.servdata.NRORG;
    }

    function getIDConsumoFromStorage(plate) {
        const localPlate = LocalPlateDataService.findPlate(plate) || {};
        return localPlate.IDCONSUMO || 'N';
    }

    this.isConsmBookmarkable = widget => {
        let dtfil = widget.dataSourceFilter.find(f => f.name === 'DTCARDAPIO').value;
        dtfil = dtfil === "@moment().format('DD/MM/YYYY')@" ? moment() : moment(dtfil, 'DD/MM/YYYY');
        let notToday = dtfil.format('YYYYMMDD') >= moment().format('YYYYMMDD');

        if (widget.dataSource && widget.dataSource.data && widget.dataSource.data.length > 0 && notToday) {
            if (getLocalVar('userData').isAnon && getLocalVar('userData').isAnon === true && getLocalVar('savedSrvCons')) {
                if (getLocalVar('savedSrvCons').length) {
                    let list = getLocalVar('savedSrvCons');
                    list.forEach(service => {
                        // jshint ignore:start                            
                        widget.dataSource.data.forEach(serv => {
                            serv.DTCARDAPIO = dtfil.format('DD/MM/YYYY');
                        });
                        let data = widget.dataSource.data.find(sc => sc.CDFILIAL == service.CDFILIAL && sc.CDSERVFILI == service.CDSERVFILI && sc.CDTIPOCARD == service.CDTIPOCARD && sc.DTCARDAPIO == service.DTCARDAPIO && sc.NRORG == service.NRORG);
                        if (data) {
                            data.IDCONSUMO = service.IDCONSUMO;
                            data.NMUSUARIO = service.NMUSUARIO;
                            data.HRCONSUMO = service.HRCONSUMO;
                        }
                        // jshint ignore:end
                    });
                }
            }
        }
        // }
    };

    this.checkIfPlateCONCFMES = (plate) => {
        let savedIDCONCNFMES = getLocalVar('IDCONCNFMESLIST');
        let retorno = false;
        if (savedIDCONCNFMES) {
            savedIDCONCNFMES.forEach(sicm => {
                if (plate.IDCONCNFMES == 'S' && plate.NRCARDAPIO == sicm.NRCARDAPIO && plate.CDTIPOPRAT == sicm.CDTIPOPRAT && plate.CDPRATO != sicm.CDPRATO) {
                    retorno = true;
                }
            });
        }
        return retorno;
    };

    this.isRatable = (widget) => {
        if (widget.dataSource && widget.dataSource.data && widget.dataSource.data.length > 0) {
            for (let i = 0; i < widget.dataSource.data.length; i++) {
                let plate = widget.dataSource.data[i];
                addServDataToPlate(plate, widget);
                let rateReturn = ['', false];
                let dtfil = widget.dataSourceFilter.find(f => f.name === 'DTCARDAPIO').value;
                dtfil = dtfil === "@moment().format('DD/MM/YYYY')@" ? moment() : moment(dtfil, 'DD/MM/YYYY');

                plate.DTCARDAPIOFormated = dtfil.format('DD/MM/YYYY');
                if (getLocalVar('userData').isAnon && getLocalVar('userData').isAnon === true) {
                    const plateForLocalStorageLookUp = Object.assign({}, plate, {DTCARDAPIO: dtfil.format('DD/MM/YYYY')});
                    let isRatable = !LocalPlateDataService.isPlateRated(plateForLocalStorageLookUp);

                    if (isRatable) {
                        if (dtfil.isSameOrBefore(moment(), 'days')) {
                            rateReturn = ['R'];
                        } else {
                            rateReturn = ['NT', 'MSG_CANT_RANK_PLATE_TODAY'];
                        }
                    } else {
                        rateReturn = ['AR', 'MSG_PLATE_ALREADY_RANKED'];
                    }

                    plate.IDCONSUMO = getIDConsumoFromStorage(plateForLocalStorageLookUp);
                } else {
                    if (plate.RESPONDIDO === 'N') {
                        if (dtfil.isSameOrBefore(moment(), 'days')) {
                            rateReturn = ['R'];
                        } else {
                            rateReturn = ['NT', 'MSG_CANT_RANK_PLATE_TODAY'];
                        }
                    } else {
                        rateReturn = ['AR', 'MSG_PLATE_ALREADY_RANKED'];
                    }
                }

                plate.rateReturn = rateReturn;
                plate.bookMark = dtfil.isSameOrAfter(moment(), 'days');
                plate.NRLOCALPESQ = getLocalVar('NRLOCALPESQ') || null;

                widget.dataSource.data[i] = plate;
                
            }
        }
    };

    this.doTheRate = (plate, widget, e) => {
        if (plate.rateReturn && plate.rateReturn[0] === 'AR' || plate.rateReturn[0] === 'NT') {
            MessageService.showMessageByCode(plate.rateReturn[1]);
        } else {
            widget.plateToRate = plate;
        }
        e.stopPropagation();
    };

    this.openBookMark = (data, widget, e) => {
        const currentPlate = data;
        const plates = widget.dataSource.data;
        let $scope = angular.element(document.getElementsByTagName("body")).scope();
        
        if(!data.DTCARDAPIO){
            let dtCard = widget.dataSourceFilter.find(f=>f.name=="DTCARDAPIO").value;

            if(dtCard[0]=="@" && dtCard[dtCard.length-1]=="@"){
                data.DTCARDAPIO = $scope.$eval(dtCard.slice(1,-1), Object.assign({}, ApplicationContext, {
                    args: {
                        owner: $scope.widget
                    },
                    moment : moment
                }));
            } else {
                data.DTCARDAPIO = dtCard;
            }    
        } else {
            data.DTCARDAPIO = data.DTCARDAPIO ? data.DTCARDAPIO : moment().format("DD/MM/YYYY");
        } 

        e.stopPropagation();
        widget.dataBookmarking = data;
        widget.dataBookmarking.NMUSUARIO = getLocalVar('userData').isAnon == false ? getLocalVar('userData').NAME : null;
        widget.dataBookmarking.nameIsFixed = getLocalVar('userData').isAnon == false;

        const {previousBookmarkDetails} = widget;
        let {CDSERVFILI, DTCARDAPIO} = data;
        
        templateManager.updateTemplate();

        DTCARDAPIO = DTCARDAPIO.includes('moment') ? moment().format('DD/MM/YYYY') : DTCARDAPIO;
        
        if(previousBookmarkDetails && previousBookmarkDetails.CDSERVFILI === CDSERVFILI && 
            previousBookmarkDetails.DTCARDAPIO === DTCARDAPIO){
            widget.dataBookmarking.HRCONSUMO = previousBookmarkDetails.HRCONSUMO;
            widget.dataBookmarking.NMUSUARIO = previousBookmarkDetails.NMUSUARIO;
        }
    };

    this.confirmBookMark = (data, widget, e) => {
        const serviceData = (widget.dataBookmarking.HORAS_LIMITE && widget.dataBookmarking) || (widget.currentRow.servdata || widget.dataBookmarking);

        let now = moment();
        let daysToReserve = Number(serviceData.NRLIMSEGPEDIDO);
        var insertPlate = false;
        
        let hoursToReserve = 0;
        if(serviceData.HRLIMSEGPEDIDO){
            hoursToReserve = serviceData.HRLIMSEGPEDIDO.split('');
            hoursToReserve.splice(2,0,':');
            hoursToReserve = moment.duration(hoursToReserve.join('')).asHours();
        }

        
        const formatedDTCARDAPIO = serviceData.DTCARDAPIO.includes('moment') ? moment().format('DD/MM/YYYY') : serviceData.DTCARDAPIO;
        let reservationTime = moment(formatedDTCARDAPIO + ' ' + widget.dataBookmarking.HRCONSUMO, 'DD/MM/YYYY HH:mm');
        if (Array.isArray(data)) {
            data.forEach((plate) => {
                plate.DSCONSUMO = widget.dataBookmarking.DSCONSUMO || null;
                plate.HRCONSUMO = widget.dataBookmarking.HRCONSUMO ? widget.dataBookmarking.HRCONSUMO.replace(':', '') : null;
                plate.DTCARDAPIO = formatedDTCARDAPIO;
                plate.NMUSUARIO = widget.dataBookmarking.NMUSUARIO;
            });
            insertPlate = true;
        } else {
            data.DSCONSUMO = data.DSCONSUMO || null;
            data.HRCONSUMO = data.HRCONSUMO ? data.HRCONSUMO.replace(':', '') : null;
            widget.dataBookmarking.DTCARDAPIO = formatedDTCARDAPIO;
        } 

        let endServiceTime = null;
        if (serviceData.HRFINAL && serviceData.HRFINAL != ":") {
            endServiceTime = moment(formatedDTCARDAPIO + ' ' + serviceData.HRFINAL, 'DD/MM/YYYY HH:mm');
        }

        let minTime = null;
        if (serviceData.HRINICIO && serviceData.HRINICIO != ":") {
            minTime = moment(formatedDTCARDAPIO + ' ' + serviceData.HRINICIO, 'DD/MM/YYYY HH:mm');
        }

        if(widget.dataBookmarking.NMUSUARIO && widget.dataBookmarking.HRCONSUMO){
            if (minTime && reservationTime.isSameOrAfter(minTime)) {
                minTime.seconds(0);
                minTime.subtract(daysToReserve, 'days');
                minTime.subtract(hoursToReserve, 'hours');

                if (now.isBefore(minTime)) {
                    if (!endServiceTime || (endServiceTime && reservationTime.isSameOrBefore(endServiceTime))) {
                        if (insertPlate) sf.operationDataPlate('MyMenuCardapioController.saveConsData()', widget, e);
                        else sf.bookMark(data, widget, e);
                        if(widget.name === 'plates'){
                            setLocalVar('HRCONSUMO', widget.dataBookmarking.HRCONSUMO);
                            setLocalVar('DSCONSUMO', widget.dataBookmarking.DSCONSUMO);
                            const userData = getLocalVar('userData');
                            widget.previousBookmarkDetails = {
                                HRCONSUMO:widget.dataBookmarking.HRCONSUMO, 
                                NMUSUARIO: userData.isAnon ? widget.dataBookmarking.NMUSUARIO : userData.NAME, 
                                DTCARDAPIO:formatedDTCARDAPIO, 
                                CDSERVFILI:widget.dataBookmarking.CDSERVFILI
                            };
                        }
                    } else {
                        MessageService.showMessageByCode('MSG_CANT_DO_RESERVE_INVALID_TIME', {PARAM: endServiceTime.format('HH:mm')}, {type:'error'});
                        widget.dataBookmarking = null;
                        widget.reload();
                    }
                } else {
                    MessageService.showMessageByCode('MSG_CANT_DO_RESERVE_INVALID_DATA', {PARAM: minTime.format('DD/MM/YYYY HH:mm')}, {type:'error'});
                    widget.dataBookmarking = null;
                    widget.reload();
                }

            } else if (!minTime) {
                now.seconds(0);
                now.add(daysToReserve, 'days');
                now.add(hoursToReserve, 'hours');

                if (reservationTime.isSameOrAfter(now)) {
                    if (insertPlate) sf.operationDataPlate('MyMenuCardapioController.saveConsData()', widget, e);
                    else sf.bookMark(data, widget, e);
                    if(widget.name === 'plates'){
                        setLocalVar('HRCONSUMO', widget.dataBookmarking.HRCONSUMO);
                        setLocalVar('DSCONSUMO', widget.dataBookmarking.DSCONSUMO);
                        const userData = getLocalVar('userData');
                        widget.previousBookmarkDetails = {
                            HRCONSUMO:widget.dataBookmarking.HRCONSUMO, 
                            NMUSUARIO: userData.isAnon ? widget.dataBookmarking.NMUSUARIO : userData.NAME, 
                            DTCARDAPIO:formatedDTCARDAPIO, 
                            CDSERVFILI:widget.dataBookmarking.CDSERVFILI
                        };
                    }
                } else {
                    MessageService.showMessageByCode('MSG_CANT_DO_RESERVE_TIME_BIGGER_THAN', {PARAM: now.format('DD/MM/YYYY HH:mm')}, {type:'error'});
                    if (insertPlate) {
                        widget.doReservation = false;
                    } else {
                        widget.dataBookmarking = null;
                        widget.reload();
                    }
                }
            } else {
                MessageService.showMessageByCode('MSG_CANT_DO_RESERVE_SERVICE_WILL_START', {PARAM: minTime.format('HH:mm')}, {type:'error'});
                if (insertPlate) {
                    widget.doReservation = false;
                } else {
                    widget.dataBookmarking = null;
                    widget.reload();
                }
            }
        } else {
            MessageService.showMessageByCode('MSG_FILL_THE_FIELD', null, {type:'error'});
        }
    };

    this.bookMark = (data, widget, e) => {
        e.stopPropagation();
        data.IDCONSUMO = data.IDCONSUMO === 'S' ? 'N' : 'S';
        if (data.IDCONSUMO === 'S') {
            MessageService.successNotificationMessageByCode('MSG_CONFIRMED_RESERVATION');
        }

        let methodToEval = 'MyMenuCardapioController.saveSrvConsData()';
        

        if(data.IDCONSUMO === 'N') {
            let clickedTime = moment();
            let limitTime = this.getLimitTime(data, widget);
            if (clickedTime.format('DD-MM-YYYY HH:mm') <= limitTime.format('DD-MM-YYYY HH:mm')) {
                widget.dataBookmarking = angular.copy(data);
                widget.dataBookmarking.NRLOCALPESQ = getLocalVar('NRLOCALPESQ') || null;
                let $scope = angular.element(document.getElementsByTagName("body")).scope();
                $scope.$eval(methodToEval, Object.assign({}, ApplicationContext, {
                    args: {
                        owner: $scope.widget
                    }
                }));
            } else {
                ScreenService.showMessage("Horário de cancelamento ultrapassado!", 'error');
                data.IDCONSUMO = 'S';
            }
        } else {
            widget.dataBookmarking = angular.copy(data);
            widget.dataBookmarking.NRLOCALPESQ = getLocalVar('NRLOCALPESQ') || null;
            let $scope = angular.element(document.getElementsByTagName("body")).scope();
            $scope.$eval(methodToEval, Object.assign({}, ApplicationContext, {
                args: {
                    owner: $scope.widget
                }
            }));
        }
    };

    this.operationDataPlate = (data, widget, e) => {
        e.stopPropagation();
        let $scope = angular.element(document.getElementsByTagName("body")).scope();
        $scope.$eval(data, Object.assign({}, ApplicationContext, {
            args: {
                owner: $scope.widget
            }
        }));
    };

    this.bookMarkPlate = (data, widget, e) => {
        e.stopPropagation();
        data.IDCONSUMO = 'S';

        widget.dataBookmarking = angular.copy(data);
        widget.dataBookmarking.NRLOCALPESQ = getLocalVar('NRLOCALPESQ') || null;
        if (widget.bookMarking == undefined) widget.bookMarking = [];
        widget.bookMarking.push(widget.dataBookmarking);
    };

    this.getLimitTime = (data, widget) => {
        let mom = null;
        let objectWidget = null;
        let dtReserva = '';
            
        if(widget.name === 'plates') {
            objectWidget = widget.currentRow.servdata;
            const isAnon = getLocalVar('userData').isAnon ? getLocalVar('savedCons').filter((plate) => plate.CDPRATO == data.CDPRATO) : [];
            if (isAnon.length > 0) {
                dtReserva = isAnon[0].DTCARDAPIO + ' ' + isAnon[0].HRCONSUMO.substring(0, 2) + ':' + isAnon[0].HRCONSUMO.substring(2, 4);
            } else {
                dtReserva = data.DTRESERVA;
            }
        } else {
            objectWidget = data;
            const isAnon = getLocalVar('userData').isAnon ? getLocalVar('savedSrvCons').filter((service) => service.CDSERVFILI == data.CDSERVFILI) : [];
            if (isAnon.length > 0) {
                dtReserva = isAnon[0].DTCARDAPIO + ' ' + isAnon[0].HRCONSUMO.substring(0, 2) + ':' + isAnon[0].HRCONSUMO.substring(2, 4);
            } else {
                dtReserva = data.DTRESERVA;
            }
        }

        let timeObject = {
            'hours': objectWidget.HRLIMSEGPEDIDO == null ? '00' : objectWidget.HRLIMSEGPEDIDO.substring(0, 2),
            'minutes': objectWidget.HRLIMSEGPEDIDO == null ? '00' : objectWidget.HRLIMSEGPEDIDO.substring(2, 4)
        };

        if (objectWidget.HRINICIO === ':') {
            mom = moment(data.DTRESERVA, 'DD-MM-YYYY HH:mm');
        } else {
            const hrInicio = dtReserva.substring(0, 10) + ' ' + objectWidget.HRINICIO;
            mom = moment(hrInicio, 'DD-MM-YYYY HH:mm');
        }
        
        mom.subtract(timeObject);
        mom.subtract(objectWidget.NRLIMSEGPEDIDO, 'days');
        return mom;
    };

    this.cancelBookMarkPlate = (data, widget, e) => {
        e.stopPropagation();
        widget.dataBookmarking = angular.copy(data);
        widget.dataBookmarking.NRLOCALPESQ = getLocalVar('NRLOCALPESQ') || null;

        if (widget.bookMarking && widget.bookMarking.length > 0) {
            const dataExistInArray = widget.bookMarking.filter((value) => value.CDPRATO == widget.dataBookmarking.CDPRATO);
            if(dataExistInArray.length > 0) {
                data.IDCONSUMO = 'N';
                widget.bookMarking.splice(widget.bookMarking.indexOf(dataExistInArray[0]), 1);
            }
        }
    };

    this.cancelReservation = (widget, e) => {
        const cancelPlate = widget.dataSource.data.filter((data) => data.IDCONSUMO == 'S');
        let clickedTime = moment();
        let limitTime = this.getLimitTime(cancelPlate[0], widget);
        if (clickedTime.format('DD-MM-YYYY HH:mm') <= limitTime.format('DD-MM-YYYY HH:mm')) {
            ScreenService.confirmMessage('Deseja cancelar a reserva?', "question", () => {
                this.operationDataPlate('MyMenuCardapioController.deleteConsData()', widget, e);
            });
        } else {
            ScreenService.showMessage("Horário de cancelamento ultrapassado!", 'error');
        }
    };

    this.doReservation = (widget, e) => {
        widget.dataBookmarking.NMUSUARIO = getLocalVar('userData').isAnon == false ? getLocalVar('userData').NAME : null;
        widget.dataBookmarking.nameIsFixed = getLocalVar('userData').isAnon == false;
        widget.doReservation = true;
        e.stopPropagation();
    };

    this.hasReservation = (widget, e ) => {
        const hasReservation = widget.dataSource.data.filter((data) => data.IDCONSUMO === 'S');
        return hasReservation.length > 0;
    };
    
    this.isSelectable = (currentElement, data) => {
        
        if(getLocalVar('userData').isAnon){
            data = getLocalVar("savedCons") || [];
        }
        
        if(currentElement.IDOPCCNFMES === 'N' && currentElement.IDCONCNFMES === 'S'){
            const plateType = currentElement.CDTIPOPRAT;
            return !data.some(d => d.CDTIPOPRAT === plateType && d.IDCONSUMO === 'S' && 
                                    d.IDOPCCNFMES === 'N' && d.IDCONCNFMES === 'S');
        } else {
            return true;
        }
    };
    
    this.getDtCardapioMoment = (widget) => {
        let dtCard = widget.dataSourceFilter.find(f=>f.name=="DTCARDAPIO").value;

        if(dtCard[0]=="@" && dtCard[dtCard.length-1]=="@"){
            let $scope = angular.element(document.getElementsByTagName("body")).scope();
            dtCard = $scope.$eval(dtCard.slice(1,-1), Object.assign({}, ApplicationContext, {
                args: {
                    owner: $scope.widget
                },
                moment : moment
            }));   
        }

        if(!dtCard.includes('moment')){
            dtCard = moment(dtCard, "DD/MM/YYYY");
        }

        return dtCard;
    };   
    
    this.isUserAnon = () => getLocalVar('userData').isAnon;

    this.isBookmarked = (plate) => {
        const cachedBookmarks = getLocalVar("savedCons") || [];

        return cachedBookmarks.some(bm=>{
            return bm.CDPRATO == plate.CDPRATO &&
                bm.CDSERVFILI == plate.CDSERVFILI &&
                bm.CDFILIAL == plate.CDFILIAL &&
                bm.DTCARDAPIO == plate.DTCARDAPIOFormated && 
                bm.IDCONSUMO === 'S';
        });
    };
    
    this.initialData = (dataBookmarking) => {
        dataBookmarking.DSCONSUMO = getLocalVar('DSCONSUMO') != undefined ? getLocalVar('DSCONSUMO') : '';
        dataBookmarking.HRCONSUMO = getLocalVar('HRCONSUMO') ? getLocalVar('HRCONSUMO').substring(0, 2) + ':' + getLocalVar('HRCONSUMO').substring(2, 4) : '';
    };
}

// FILE: ./assets/templates/controllers/ImageContainerController.js
function ImageContainerController() {
    this.getAvailableHeight = function (container) {
        var imageContainerSize = container.imageContainerSize || "1/1";

        var headerSize = container.showHeader ? "40px" : "0px";
        var footerSize = container.showFooter ? "56px" : "0px";

        return "calc((100vh - (" + headerSize + " + " + footerSize + ")) * " + imageContainerSize + ")";
    };

    this.getLogoForHome = (container) => {
        if (getLocalVar('qrCodeURL')) {
            container.hasImageLogo = true;
            return window.fileServerPath + getLocalVar('qrCodeURL');
        } else {
            container.hasImageLogo = false;
            return container.imageSource;
        }
    };
}

// FILE: ./assets/templates/controllers/InformativoController.js
function InformativoController($scope, ScreenService) {
    let sf = this;
    
    this.init = () => {
        $scope.widget.goToDetails = this.goToDetails;
    };

    this.isValid = (infor) => {
        return (moment()._d <= moment(infor.DTEXPIRA, 'DD/MM/YYYY')._d || infor.DTEXPIRA === null) && 
                (infor.IDEXIBICAO === 'M' || infor.IDEXIBICAO === 'A');
    };
    this.limitText = (text) => {
        let retorno = text.replace(/&nbsp;/gi,'').trim();
        return retorno.length > 68 ? retorno.substr(0, 68) +'...' : retorno;
    };
    
    this.goToDetails = (servdata, widget) => {
        let widgets = widget.container.widgets;
        let acwidget = widgets.indexOf(widget);

        if (widgets[acwidget + 1]) {
            widget.isVisible = false;

            let widgetToGo = widgets[acwidget + 1];
            let newtitle = servdata[widgetToGo.label];
            widgetToGo.container.label = newtitle.length > 30 ? newtitle.substr(0, 30)+'...' : newtitle;

            let filterToInherit = widgetToGo.filterToInherit;
            widgetToGo.dataSourceFilter = [];

            widgetToGo.isVisible = true;
            widgetToGo.currentRow.servdata = servdata;
            widgetToGo.activate();
            widgetToGo.reload();
        }
    };

    /**
     * @param {dataSourceElement} informativos 
     */
    this.shouldShowInformativos = (informativos) => 
        !(informativos.filter(informativo => this.isValid(informativo)).length)
    ;
}

// FILE: ./assets/templates/controllers/InformativoDetailController.js
function InformativoDetailController($sce) {
    this.trustAsHtml = function (htmlContent) {
        return $sce.trustAsHtml(htmlContent);
    };
}

// FILE: ./assets/templates/controllers/MyMenuSelectNativeController.js
function MyMenuSelectNativeController() {
    this.getPlataform = function () {
        var plataform = navigator.platform || null;
        var androidRegExp = /Android|Linux|null/;
        
        return androidRegExp.test(plataform) ? 'android' : 'outros';
    };
}

// FILE: ./assets/templates/controllers/QRCodeController.js
function QRCodeController(ApplicationContext) {
    var ScreenService = ApplicationContext.ScreenService;
    var templateManager = ApplicationContext.templateManager;
    var General = ApplicationContext.General;
    var ZHPromise = ApplicationContext.ZHPromise;
    let FCMBootstrap = ApplicationContext.FCMBootstrap;
    let engine = ApplicationContext.engine;
    var MessageService = ApplicationContext.MessageService;


    var self = this;

    let TColorToRGB = function (TColor) {
        var r = TColor & 0xFF;
        var g = (TColor >> 8) & 0xFF;
        var b = (TColor >> 16) & 0xFF;
        return '(' + r + ',' + g + ',' + b + ')';
    };

    this.openQRCodeInputPopup = function (container) {
        var qrcodeInputPopup = container.getWidget('qrcodeInputPopup');
        qrcodeInputPopup.newRow();
        ScreenService.openPopup(qrcodeInputPopup);
    };

    this.readQRCodeFromInputPopup = function (qrcodeInputPopup) {
        var qrCodeData = qrcodeInputPopup.currentRow.qrCodeData;

        self.readQRCode(qrCodeData).then(function () {
            ScreenService.closePopup(qrcodeInputPopup);
        });
    };

    const updateTOSURL = (httpRequestResponse) => {
        const tosURL = httpRequestResponse.dataset.tosURL && httpRequestResponse.dataset.tosURL[0];
        if(tosURL){
            setLocalVar('tosURL', tosURL);
        }
    };

    let _setNRORGONSESSION = (qrCodeDataRow) => {
        return General.callRestEngine('/setNRORGONSESSION', 'Row', qrCodeDataRow).then((response) => {
            if(response && response.dataset){
                const token = response.dataset.JWT[0];
                setLocalVar('token', token);
            }
        });
    };

    let _setLocalVarsFromQRCodeData = (qrCodeDataRow, url, porta, isApp) => {
        return General.callRestEngine('/readQRCode', 'Row', qrCodeDataRow).then(
            function (response) {
                let qrCodeResp = response.dataset.qrcode;
                let fileServerURL = response.dataset.fileServerURL[0];
                localStorage.setItem('fileServerURL', fileServerURL);
                window.fileServerPath = fileServerURL;

                setLocalVar('IDPERANON', qrCodeResp.IDPERANON);
                setLocalVar('IDEXIBENUTR', qrCodeResp.IDEXIBENUTR);
                setLocalVar('IDTAMCARDIMG', qrCodeResp.IDTAMCARDIMG);
                setLocalVar('DSCORPRIMARIA', qrCodeResp.DSCORPRIMARIA || '#E63635');
                if (qrCodeResp.DSCORPRIMARIA) {
                    document.documentElement.style.setProperty('--secundaryColor', 'rgb' + TColorToRGB(qrCodeResp.DSCORPRIMARIA));
                } else {
                    document.documentElement.style.setProperty('--secundaryColor', getLocalVar('DSCORPRIMARIA'));
                }

                setLocalVar('qrCodeURL', qrCodeResp.URLLOGO);
                setLocalVar('NRLOCALPESQ', qrCodeResp.NRLOCALPESQ);
                setLocalVar('DSNOMEEXIBICAO', qrCodeResp.DSNOMEEXIBICAO);
                setLocalVar('customerID', qrCodeResp.CUSTOMERID);
                setLocalVar('validQRCode', true);
                setLocalVar('backendURL', url);

                const token = response.dataset.JWT[0];
                setLocalVar('token', token);
                updateTOSURL(response);
                
                if(isApp) {
                    ScreenService.openWindow('product#login');
                }
            });
    };

    let _onQRCodeValidationError = (error) => {
        return new Promise((resolve, reject) => {
            setLocalVar('validQRCode', false);
            var errorMessage = 'MSG_ERRO_VALID_QRCODE';
            if (error.data && error.data.error) {
                var errorData = error.data.error;
                errorMessage = errorData;
            }
            
            MessageService.showMessageByCode(errorMessage, null, {type: 'error'});

            reject();
        });
    };
    
    let loginAnonimous = () => {
        General.callRestEngine("/anonimousAccount", "Row", {}).then(function (response) {
            if(response.dataset){
                const token = response.dataset.JWT[0];
                setLocalVar('token', token);
            }
            setLocalVar('userData', response.dataset.user);
            let fcmToken = localStorage.getItem('fcmToken');
            engine.sendFCMTokenSaveRequest(fcmToken, true).catch(() => engine.sendFCMTokenSaveRequest(fcmToken, false));
            ScreenService.openWindow('product#home');
        }, function (response) {
            var errorMsg = "Erro";
            if (response.data && response.data.error) {
                errorMsg = response.data.error;
            }

            ScreenService.showMessage(ScreenService.i18n(errorMsg), 'E');
        });
    };
    
    this.readQRCode = function (qrCodeData) {
        // let promise = ZHPromise.defer();
        let dados = [];
        
        if(qrCodeData.search("params") > 0) {
            dados = qrCodeData.split('/mobile/#/product#login?params=');
            return this.qrCodeWeb(dados).then(() => {
                loginAnonimous();
            });
        } else {
            dados = qrCodeData.split(';');
            return qrCodeApp(dados);
        }
    };
    
    let qrCodeApp = (dados) => {
        if (dados.length !== 4) {
            _showErrorMessageCode();
        } else {
            let dns = dados[0];
            const porta = dados[1];
            let nrqrcode = dados[2];
            let nrorg = dados[3];
            
            return _setDataAfterQrCode(dns, nrqrcode, nrorg, porta, true);
        }
    };
    
    this.qrCodeWeb = function(dados) {
      if(dados.length !== 2) {
        _showErrorMessageCode();
      } else {
        let dns = dados[0];
        let cript = dados[1];
        
        return _getDecriptedUrlData(cript).then((decriptedData) => {
            if(decriptedData.length > 0) {
                let porta = decriptedData[0].DSPORTA;
                let nrqrcode = decriptedData[0].NRQRCODELOCALPESQ;
                let nrorg = decriptedData[0].NRORG;
                return _setDataAfterQrCode(dns, nrqrcode, nrorg, porta, false);    
            } else {
                _showErrorMessageCode();
            }
        }).catch(_showErrorMessageCode);
      } 
    };
    
    let _showErrorMessageCode = () => {
        MessageService.showMessageByCode('MSG_INVALID_QRCODE', null, {type: 'error'});
        return Promise.reject();
    };
    
    let _setDataAfterQrCode = (dns, nrqrcode, nrorg, porta, isApp) => {
        let qrCodeDataRow = {
            'nrqrcode': nrqrcode,
            'nrorg': nrorg
        };

        setLocalVar('qrCodeData', qrCodeDataRow);
        setLocalVar('NRORG', nrorg);

        let url = dns + window.qrcodebaseURL;
        let urlHTTPS = url.replace('http:', 'https:');
        templateManager.updateURL(url);

        return new Promise((resolve, reject) => {
            _setNRORGONSESSION(qrCodeDataRow).then(() => {
                _setLocalVarsFromQRCodeData(qrCodeDataRow, url, porta, isApp)
                    .then(() => resolve())
                    .catch((errorArg) => {
                        _onQRCodeValidationError(errorArg).catch(reject);
                    });
            }).catch(() => {
                templateManager.updateURL(urlHTTPS);
                _setNRORGONSESSION(qrCodeDataRow).then(() => {
                    _setLocalVarsFromQRCodeData(qrCodeDataRow, urlHTTPS, porta, isApp);
                    resolve();
                }).catch((errorArg) => {
                    _onQRCodeValidationError(errorArg).catch(reject);
                });
            });
        });
    };
    
    let _getDecriptedUrlData = (cript) => {
        return General.callRestEngine('/decriptUrlData', 'Row', { 'criptedData': cript }).then((response) => {
            if(response && response.dataset){
                return response.dataset.decriptedData;
            }
        });
    };

    // let bindVisibilityAccessFunctions = function bindVisibilityAccessFunctions() {
    //     Object.defineProperty($scope.widget, 'isVisible', {
    //         set: (newValue) => {
    //             this._isVisible = newValue;
    //         },
    //         get: () => this._isVisible,
    //         // enumerable: true,
    //         // configurable: true
    //     });
    // };
    // // constructor(){
    // // this.$scope = $scope;
    // this._isVisible = $scope.widget._isVisible;
    // bindVisibilityAccessFunctions();
    // }

    let startScan = () =>
        new Promise((resolve, reject) => {
            if ('QRScanner' in window) {
                QRScanner.prepare(() => {
                    QRScanner.scan((err, text) => {
                        if (err) {
                            reject(err);
                        } else {
                            resolve(text);
                        }
                        endScan();
                    });
                    QRScanner.show();
                });
            } else {
                reject('QRScanner is not avaliable');
            }
        });


    let endScan = () => {
        if (QRScanner) {
            // webView.isViisible = true;
            QRScanner.destroy();
        }
    };

    let requestPermission = () =>
        new Promise((resolve, reject) => {
            if (QRScanner) {
                QRScanner.prepare((err, status) => {
                    if (err) {
                        reject(err);
                    }
                    if (status.authorized) {
                        resolve(true);
                    } else if (status.denied) {
                        resolve(false);
                    } else {
                        resolve(false);
                        // needs better support
                        // we didn't get permission, but we didn't get permanently denied. (On
                        // Android, a denial isn't permanent unless the user checks the "Don't
                        // ask again" box.) We can ask again at the next relevant opportunity.
                    }
                });
            } else {
                reject('QRScanner is not avaliable');
            }
        });

    this.backAndCancelEvent = function (widget) {
        widget.isScaning = false;
        document.body.style.background = '';
        if ('QRScanner' in window) {
            QRScanner.getStatus((status) => {
                if (status.scanning && status.showing) {
                    QRScanner.destroy();
                }
            });
        }
    };

    this.backAndCancel = function (action, widget) {
        if (action) {
            action.isVisible = false;
        }
        widget.isScaning = false;
        document.body.style.background = '';
        if ('QRScanner' in window) {
            QRScanner.getStatus((status) => {
                if (status.scanning && status.showing) {
                    QRScanner.destroy();
                }
            });
        }
    };

    this.showQRCodeScanner = (widget) => {
        widget.isScaning = true;
        widget.actions.find(ac => ac.name === 'cancel').isVisible = false;
        if ('QRScanner' in window) {
            document.body.style.background = 'transparent';
            startScan().then((text) => {
                self.readQRCode(text)
                    .catch(() => this.showQRCodeScanner(widget))
                    .then(() => FCMBootstrap.start());
                document.body.style.background = '';
            }).catch((error) => {
                MessageService.showMessageByCode('MSG_INVALID_QRCODE', null, {type: 'error'});
                this.showQRCodeScanner(widget);
                document.body.style.background = '';
            });
            let fixBackground = () => {
                this.backAndCancelEvent(widget);
                window.removeEventListener('popstate', fixBackground);
            };
            window.addEventListener('popstate', fixBackground, false);
        }
    };

}

Configuration(function (ContextRegister) {
    ContextRegister.register('QRCodeController', QRCodeController);
});

// FILE: ./assets/templates/controllers/SelectAutocompleteMultipleController.js
// jshint ignore: start
'use strict';
function SelectAutocompleteMultipleController(
    $scope,
    $element,
    ScreenService,
    ZhUtil,
    maskEngine,
    restEngine,
    $q,
    ZhLogger,
    ApplicationContext
    ){

    
    /**
     * @type {Array<String>}
     */
    let keys = [];
    /**
     * @type {Array<Object>}
     */
    $scope.fieldRow = $scope.fieldRow || [];
    /**
     * @type {Object.<String, Object>}
     */
    $scope.field.selectedElements = $scope.field.selectedElements || {};
    // $scope.field

    // console.log(selectedElements);
    /**
     * @type {Object}
     */
    var events = {};
    // document.querySelector('.zh-notification-messages').style.display = 'none';
    var activeRequests = [];

    /**
     * @type {Object}
     * a object that will precache the results of each query
     */
    var cacheBase = {};
    /**
     * the spinner element that indicates progress   
     * @type {HTMLElement}
     */
    var spinner = null;
    /**
     * a reference to the current highlited element of the list   
     * @type {HTMLElement}
     */
    var currentHighletedListElement = null;
    /**
     * @type {Object[]} 
     */
    var filteredAditionalOptions = $scope.field.aditionalOptions || [];
    /**
     * @type {Object[]} 
     */
    var filteredListContents = [];
    /** it will be initialized on the initializeData method
     * @type {listContents}
     */
    var listContents = null;
    /**
     * @type {HTMLElement}
     */
    var rootElement;
    /**
     * @type {HTMLElement}
     */
    var popUp;
    /**
     * @type  {HTMLElement}
     */
    var listElement;
    /**
     * @type {HTMLElement}
     */
    var inputElement;
    /**
     * @type {HTMLElement}
     */
    var zhLoader;
    /**
     * @type {HTMLElement}
     */
    var autoCompletePreviewElement;
    /**
     * @type {HTMLElement}
     */
    var labelElement;
    // /**
    //  * @type {HTMLElement}
    //  */
    // var headerLabelElement;
    /**
     * @type {HTMLElement}
     */
    var clearButtonElement;
    /**
     * @type {Object[]}
     */
    var fieldInfo = [];

    /**
     * an element that will be used as a basis to render each list element 
     * @type {HTMLElement}
     */
    var baseListRowToClone;
    /**
     * the root element of the chips 
     * @type {HTMLDivElement}
     */
    let chipList;
    /**
     * an element for showing chips and opening the popup 
     * @type {HTMLDivElement}
     */
    let contentDisplay;

    var filterConditions = {
        name: "",
        value: "%%",
        operator: "LIKE_ALL"
    };

    var shouldPreventKeyUp = false;
    var lastUpdateTime;
    var updateRequestReference = null;
    var deferredToCancelRequest = false;

    var cacheFirstResultSet = function() {
        cacheBase[""] = listContents;
    };

    /**
     * schedule new http requests and filter the current elements
     * @param {KeyboardEvent} [event] 
     */
    var keyStrokeHandler = function(event) {
        if (event.keyCode > 45 || event.keyCode === 8) {
            //console.log('%ckeyCode: ' + event.keyCode + "   key: '" + event.key + "'", 'color: orange');
            // var begining = performance.now(); // LOG TIME
            if (activeRequests.length) {
                if (activeRequests[0].reqText === inputElement.value) {
                    return;
                }
                else {
                    cancelRequest();
                }
            }
            if (cacheBase[inputElement.value]) {
                filteredListContents = cacheBase[inputElement.value];
                renderListElements(cacheBase[inputElement.value]);
                // //console.log('rendering : ' + (performance.now() - begining)); // LOG TIME
                return;
            }
            filteredAditionalOptions = filterElements(inputElement.value, $scope.field.aditionalOptions || [], true);
            var filtered = filterElements(inputElement.value, null, true);
            renderListElements(filtered);
            // //console.log('rendering : ' + (performance.now() - begining)); // LOG TIME
            // showAutoCompleteText();
            // var timeSinceUpdate = performance.now() - lastUpdateTime;
            // if (timeSinceUpdate >= 200) {
            //     update();
            // }
            // else {
            //     updateRequestReference = setTimeout(update, 200);
            // }
            update();
        }
        shouldPreventKeyUp = true;
    };

    /**
     * handles the command keys keypress, or propagates the event chain for the keyStrokeHandler to resolve
     * @param {KeyboardEvent} [event] 
     */
    var commandKeyStrokeHandler = function(event) {
        var code = event.keyCode;
        if (code > 45 || code === 8) {
            // do nothing and let the keyStrokeHandler handle the keyup event 
            shouldPreventKeyUp = false;
            return;
        }
        else if (code === 40 /* down arrow */ ) {
            if (currentHighletedListElement.parentElement.lastChild !== currentHighletedListElement) {
                currentHighletedListElement.classList.remove('active');
                currentHighletedListElement = currentHighletedListElement.nextElementSibling;
                currentHighletedListElement.classList.add('active');
            }
        }
        else if (code === 38 /* up arrow */ ) {
            if (currentHighletedListElement.parentElement.firstChild !== currentHighletedListElement) {
                currentHighletedListElement.classList.remove('active');
                currentHighletedListElement = currentHighletedListElement.previousElementSibling;
                currentHighletedListElement.classList.add('active');
            }
        }
        else if (code === 13) /* enter */ {
            confirm(null, currentHighletedListElement);
        }
        shouldPreventKeyUp = true;
    };

    /**
     * shows the first element of the filteredListContents array
     */
    var showAutoCompleteText = function() {
        autoCompletePreviewElement.value = getFormatedString(filteredListContents[0], 0);
    };

    var refreshLastUpdateTime = function() {
        lastUpdateTime = performance.now();
    };
    var hideSpinner = function() {
        spinner.classList.add('hidden');
    };
    var onRequestComplete = function() {
        hideSpinner();
        refreshLastUpdateTime();
        //console.log('%cTHE REQUEST WAS SUCESSFULLY COMPLETED', 'color: purple;');
    };
    /**
     * @returns {Promise<null>} 
     */
    var update = function() {
        if (updateRequestReference !== null) {
            clearTimeout(updateRequestReference);
            updateRequestReference = null;
        }
        filterConditions.value = "%" + inputElement.value + "%";
        return fetchData().then(refreshLastUpdateTime);
    };

    var cancelRequest = function() {
        var numberOfCanceledRequests = activeRequests.length;
        ZhLogger.setLevel(999);
        ZhLogger.previousLevel = ZhLogger.level;
        while (activeRequests.length) {
            activeRequests.pop().resolve();
        }
        //console.log('%c' + numberOfCanceledRequests + ' requests were CANCELED ', 'color: white;background-color:red;');
    };

    var initiliazefieldInfo = function() {
        if ($scope.field.fields) {
            for (var i = 0; i < $scope.field.fields.length; i++) { // TODO: use reverse for instead
                var label = ($scope.field.fields[i].label ? ($scope.field.fields[i].label + ": ") : "");
                fieldInfo.push({
                    "label": label,
                    "name": $scope.field.fields[i].name,
                    "mask": $scope.field.fields[i].mask || false,
                    "isVisible": $scope.field.fields[i].isVisible !== false,
                });
                filterConditions.name += $scope.field.fields[i].name + '|';
            }
            filterConditions.name = filterConditions.name.substr(0, filterConditions.name.length - 1);
        }
        else {
            // var labelWithNoExtraFields = ($scope.field.label ? ($scope.field.label + ": ") : "");
            fieldInfo.push({
                "label": "" /*labelWithNoExtraFields*/ ,
                "name": $scope.field.name,
                "mask": $scope.field.mask || false
            });
            filterConditions.name = $scope.field.name;
        }
        $scope.field.dataSource.itemsPerPage = 10;
        $scope.field.dataSource.disableLoader = true;

        if($scope.field.events){
            let fieldOnChangeEvent = $scope.field.events.find(event => event.name === 'FieldOnChange');
            if(fieldOnChangeEvent){
                events.FieldOnChange = fieldOnChangeEvent.code;
            }
        }

        keys = ($scope.field.dataSource.getVO() && $scope.field.dataSource.getVO().localKey);

    };

    var initiliazeDataSetter = function() {
        if ($scope.field.dataSource) {
            Object.defineProperty($scope.field.dataSource, 'data', {
                get: function() {
                    return listContents;
                },
                set: function(newValue) {
                    if (newValue === listContents) {
                        return;
                    }
                    newValue = [...filteredAditionalOptions].concat([...newValue]);
                    filteredListContents = listContents = newValue;
                    // theGreatCacheMachine[inputElement.value] = listContents;
                    // //console.log("%cCached '" + inputElement.value + "', with length: " + listContents.length, 'color: purple');
                    renderListElements(listContents);
                    if(!$scope.field.descriptionTextRendered){
                        let selectedElement = $scope.field.dataSource.data.find(dataSourceElement => {
                            return dataSourceElement[$scope.field.name] === $scope.fieldRow;
                        });
                        if(selectedElement){
                            $scope.descriptionText = $scope.getDescriptionString(selectedElement);
                        }
                    }
                },
                enumerable: true,
                configurable: true
            });
        }

        

    };

    var doCache = function(cacheKey, cacheList) {
        cacheBase[cacheKey] = cacheList;
        //console.log("%cCached '" + cacheKey + "', with length: " + cacheList.length, 'color: purple');
    };

    var initiliazeData = function() {
        if ($scope.field.dataSource.data) {
            filteredListContents = listContents = $scope.field.dataSource.data;
            // cacheFirstResultSet();
            let arrayToRender = listContents;
            if(!filteredAditionalOptions.every(aditionalOption => listContents.includes(aditionalOption))){
                listContents = filteredAditionalOptions.concat(listContents);
            }
            renderListElements(arrayToRender);
        }
        else {
            listContents = [];
        }
        if (!$scope.field.dataSource.dataSourceFilter) {
            $scope.field.dataSource.dataSourceFilter = [];
        }
    };
    // $scope.fieldRow =

    // let initiliazeFieldRowGetSer = () => {
    //     if($scope.field.widget){
    //         $scope.__fieldRow = $scope.fieldRow || [];
    //         // $scope.field.widget.currentRow = $scope.field.widget.currentRow || [];
    //         $scope.fieldRow = $scope.field.widget.currentRow[$scope.field.name] || [];
    //         Object.defineProperty($scope, 'fieldRow', {
    //             get: function() {
    //                 return $scope.__fieldRow;
    //             },
    //             set: function(newValue) {
    //                 $scope.__fieldRow = newValue
    //             },
    //             enumerable: true,
    //             configurable: true
    //         });
    //     }
    // };

    // let initiliazeFieldRowGetSer = () => {
    //     if($scope.field.widget){
    //         $scope.field.widget.currentRow = $scope.field.widget.currentRow || {};
    //         $scope.fieldRow = $scope.field.widget.currentRow[$scope.field.name] || [];
    //         Object.defineProperty($scope.field.widget.currentRow, $scope.field.name, {
    //             get: function() {
    //                 return $scope.fieldRow;
    //             },
    //             set: function(newValue) {
    //                 $scope.fieldRow = newValue
    //             },
    //             enumerable: true,
    //             configurable: true
    //         });
    //     }
    // };

    $scope.autoCompleteConstructor = function() {
        rootElement = $element.get(0);
        listElement = rootElement.querySelector('#element-list');
        inputElement = rootElement.querySelector('#content');
        labelElement = rootElement.querySelector('.select-autocomplete-form-field-label');
        labelElement.textContent = $scope.field.label || "";
        if($scope.field.validations && $scope.field.validations.hasOwnProperty('required')){
            labelElement.classList.add('select-autocomplete-required-indicator');
        }
        // switchElement = rootElement.querySelector('#switch');
        autoCompletePreviewElement = rootElement.querySelector('#autocomplete-preview');
        clearButtonElement = rootElement.querySelector('#clear');
        clearButtonElement.addEventListener('click', clear);
        spinner = rootElement.querySelector('.select-autocomplete-spinner');
        popUp = rootElement.querySelector('.select-autocomplete-pop-up');
        chipList = rootElement.querySelector('#select-autocomplete-chips');
        contentDisplay = rootElement.querySelector('#select-autocomplete-content-display');
        inputElement.addEventListener('keyup', keyStrokeHandler);
        inputElement.addEventListener('keydown', commandKeyStrokeHandler);
        inputElement.placeholder = "Pesquise " + $scope.field.label;
        zhLoader = document.querySelector('.zh-background-loading');
        let headerLabelElement = rootElement.querySelector('.select-autocomplete-field-label');
        headerLabelElement.textContent = $scope.field.label;
        let backButton = rootElement.querySelector('#back-button');
        let confirmButton = rootElement.querySelector('#confirm-button');
        backButton.addEventListener('click', $scope.closePopUp);
        confirmButton.addEventListener('click', $scope.closePopUp);

        

        baseListRowToClone = rootElement.querySelector('.select-autocomplete-row').cloneNode(true);
        baseListRowToClone.style.display = 'flex';

        if($scope.beforeSelectAutoCompleteInitialized) {
            $scope.beforeSelectAutoCompleteInitialized($scope);
        }

        $scope.fieldRow = $scope.fieldRow || [];
        initiliazefieldInfo();
        initiliazeData();
        initiliazeDataSetter();
        // initiliazeFieldRowGetSer();
        if($scope.onSelectAutoCompleteMultipleInitialized) {
            $scope.onSelectAutoCompleteMultipleInitialized($scope);
        }


        chipList.addEventListener('click', removeChip);

        if($scope.fieldRow.length > 0){ // if coming from a refresh 
            renderChips();
        }
    };

    let removeChip = event => {
        let clickedChip = event.target;
        let id = clickedChip.getAttribute('__hashkey');
        delete $scope.field.selectedElements[id];
        // chipList.removeChild(clickedChip);
        renderChips();
        Array.from(listElement.children).forEach(listRow => {
            if(listRow.__hashKey === id) {
                listRow.classList.remove('checked');
            }
        });

        setFieldRow();
        // // if(!$scope.field.outData){
        //     $scope.fieldRow = [];
        //     for(selectedElementKey in $scope.field.selectedElements){
        //         $scope.fieldRow.push($scope.field.selectedElements[selectedElementKey][$scope.field.name]);
        //     }
        // // }
    };

    /**
     * @param {Array<Object>} listElements an array of objects to be rendered  
     */
    function renderListElements(listElements) {
        //render new list

        while (listElement.lastChild) {
            listElement.removeChild(listElement.lastChild);
        }
        for (var index = 0; index < 8 && index < listElements.length; index++) {
            var newHtmlListElement = baseListRowToClone.cloneNode(true);
            newHtmlListElement.children[1].children[0].textContent = getFormatedString(listElements[index], 0);
            newHtmlListElement.children[1].children[1].children[0].textContent = getFormatedString(listElements[index], 1);
            newHtmlListElement.children[1].children[1].children[1].textContent = getFormatedString(listElements[index], 2);

            // if(!$scope.field.selectedElements.indexOf(listElements[index])){
            //     newHtmlListElement.classList.add('checked');
            // }
            if(isSelected(listElements[index])){
                newHtmlListElement.classList.add('checked');
            }
            newHtmlListElement.realIndex = index;
            newHtmlListElement.__hashKey = getHashKey(listElements[index]);
            newHtmlListElement.addEventListener('click', confirm);
            listElement.appendChild(newHtmlListElement);
        }
        if (index > 0) {
            if (currentHighletedListElement)
                currentHighletedListElement.classList.remove('active');
            currentHighletedListElement = listElement.children[0];
            currentHighletedListElement.classList.add('active');
        }
    }


    /**
     * @param {Object} [row] - a object to extract the values for the display string
     * @returns {string} a string with the descriptionFields that are in the row concated with pipe
     */
    $scope.getDescriptionString = function(row){
        if(row){
            let descriptionFields = $scope.field.descriptionChips || $scope.field.descriptionFields || [];
            return descriptionFields.map(descriptionField => row[descriptionField]).join(' | ');
        } else {
        let descriptionFields = ($scope.field.descriptionFields || []).concat($scope.field.name);
            let descriptionString = "";
            for(selectedElementKey in $scope.field.selectedElements){
                let selectedValue = $scope.field.selectedElements[selectedElementKey];
                descriptionString += 
                    descriptionFields.map(descriptionField => selectedValue[descriptionField]).join('|') + '🔝';
            }
            return descriptionString.slice(0, -1);
        }
    }

    /**
     * returns an String with the field name concatenated, or an empty string if the object doesn't have that field  
     * @returns {String} 
     * @param {Object} object 
     * @param {Number} index the index of the field that is being searched on the fieldInfo array 
     */
    function getFormatedString(object, index) {
        if (fieldInfo.length <= index && object) {
            return "";
        }
        else if (object.hasOwnProperty(fieldInfo[index].name) && fieldInfo[index].isVisible) {
            var value = object[fieldInfo[index].name];
            if (fieldInfo[index].mask) {
                value = Util.applyMask(value, fieldInfo[index].mask, maskEngine, ZhUtil);
            }
            return fieldInfo[index].label + value;
        }
        else {
            return "";
        }
    }

    /**
     * @param {String} stringToSearch 
     * @param {Array<Object>} arraryToSearch
     * @param {Boolean} [cleanReturn] - if is set to true the method doesn't affect 'filteredListContents' and returns a new array instead 
     */
    "use strict";
    function filterElements(stringToSearch, arraryToSearch, cleanReturn) {
        var workArray;
        arraryToSearch = arraryToSearch || listContents;
        if (!cleanReturn) {
            workArray = filteredListContents;
        }
        workArray = [];
        for (var i = 0; /*i < 9 && */ i < arraryToSearch.length; i++) {
            if (stringIsInJSON(arraryToSearch[i], stringToSearch))
                workArray.push(arraryToSearch[i]);
        }
        if (cleanReturn)
            return workArray;
    }

    /**
     * //TODO refactor docs// search all properties of a json object to find a string
     * @param {Object} object 
     * @param {String} stringToMatch 
     */
    function stringIsInJSON(object, stringToMatch) {
        stringToMatch = stringToMatch.toLowerCase();
        for (var i = fieldInfo.length; i--;) {
            if (('' + object[fieldInfo[i].name]).toLowerCase().indexOf(stringToMatch) !== -1)
                return true;
        }
        return false;
    }

    /** 
     * @param {MouseEvent} event - a event to extract the list html element
     * @param {HTMLElement} listElement - a list element to select
     */
    var confirm = function(event, listElement) {
        var element;
        var shouldForceDigest = true;
        if (event) {
            element = event.currentTarget;
        }
        else {
            element = listElement;
            shouldForceDigest = true;
        }
        var newValueObject = filteredListContents[element.realIndex];
        // labelElement.classList.add('rui-txt-input-label-val');
        setValue(newValueObject, element);
        
        // inputElement.blur();
        // ApplicationContext.UIManager.goBack();
        
        if (shouldForceDigest) {
            $scope.$apply();
        }
    };


    /**
     * @param {Object} dataSourceElement a ZhRowObject to generate a key
     */
    let getHashKey = (dataSourceElement) => {
        // if the keys weren't defined use all properties as keys, poor_performance.jpg
        // todo: find a better way to hashElements
        let hashKeys = keys || Object.keys(dataSourceElement); 
        return hashKeys.map(currentKey => dataSourceElement[currentKey]).join("");
    };

    /**
     * @param {Object} dataSourceElement a ZhRowObject
     */
    let isSelected = (dataSourceElement) => getHashKey(dataSourceElement) in $scope.field.selectedElements;

    
    let renderChips = function(){
        let selectedElementsKeys = Object.keys($scope.field.selectedElements);
        let elementChips = selectedElementsKeys.map(selectedElement => `
            <span class="select-autocomplete-chip" 
                __hashKey="${getHashKey($scope.field.selectedElements[selectedElement])}"
            > 
                ${$scope.getDescriptionString($scope.field.selectedElements[selectedElement])}
                <i class="material-icons remove-icon"> removecircle </i>
            </span>
        `).join("");
        chipList.innerHTML = elementChips;
        let addAction = `<span class="select-autocomplete-chip ${selectedElementsKeys.length ? 'accent' : ''} " ng-click="openPopUp()"> 
            <i class="material-icons select-autocomplete-chip-action"> addcircle </i>
            ${selectedElementsKeys.length ? '' : 'Adicionar'} 
        </span>
        `;

        contentDisplay.innerHTML = addAction + chipList.innerHTML;
        contentDisplay.scrollTo(0,0); // rtl trick

        // if($scope.elementToMountChips){
        //     $scope.elementToMountChips.innerHTML = chipList.outerHTML;
        //     // let newChipList = $scope.elementToMountChips.querySelector('#select-autocomplete-chips');
        //     // newChipList.addEventListener('click', removeChip);
        // }
    };

    let getValueWithOutData = (row) => {
        let valueWithOutData = {};
        const field = $scope.field;
        
        for (const key in field.outData) {
            const outDataProperty = field.outData[key];
            valueWithOutData[key] = row[outDataProperty];
        }

        const keyToExtractValueField = $scope.field.valueField || $scope.field.name;
        valueWithOutData[keyToExtractValueField] = row[keyToExtractValueField];

        return valueWithOutData;
    };

    let setFieldRow = () => {
        let tempArray = [];
        if(!$scope.field.outData){
            // $scope.fieldRow = []; // todo remove this line's comment
            // todo: remove this line
            for(selectedElementKey in $scope.field.selectedElements){
                const keyToExtractValue = $scope.field.valueField || $scope.field.name;
                tempArray.push($scope.field.selectedElements[selectedElementKey][keyToExtractValue]);
            }
            // $scope.fieldRow = $scope.field.selectedElements.map(selectedElement => selectedElement[$scope.field.name]);
        } else {
            for(selectedElementKey in $scope.field.selectedElements){
                const row = $scope.field.selectedElements[selectedElementKey];
                tempArray.push(getValueWithOutData(row));
            }
        }
        $scope.fieldRow = tempArray;
    };

    /**
     * refreshes the ui, changes internal variables and trigger listeners
     * TODO: refactor this doc
     * @param {Object} newValue 
     * @param {HTMLElement} [element] 
     */
    let setValue = function(newValue, element){
        if(!isSelected(newValue)) {
            // delete newValue.__isSelected;
            // delete newValue.__is_new;
            $scope.field.selectedElements[getHashKey(newValue)] = newValue;
            if(element) {
                element.classList.add('checked');
            }
        } else {
            // let index = $scope.field.selectedElements.indexOf(newValue);
            // $scope.field.selectedElements.splice(index, 1);
            delete $scope.field.selectedElements[getHashKey(newValue)];
            if(element) {
                element.classList.remove('checked');
            }
        }

        // this may be useful in the future
        // if(Object.keys($scope.field.selectedElements).length > 0){
        //     clearButtonElement.style.display = 'initial';
        // }

        setFieldRow();        

        // inputElement.value = $scope.descriptionText = $scope.getDescriptionString(newValue);
        
        if($scope.onSelectAutoCompletedValueChanged){
            $scope.onSelectAutoCompletedValueChanged(newValue, $scope.field);
        }

        if(events.FieldOnChange) {
            $scope.$eval(events.FieldOnChange, Object.assign({}, ApplicationContext, {
                args: {
                owner: {field: $scope.field},
                row: $scope.field.widget.currentRow
                }
            }));
        }

        renderChips();
    }

    /**
     * clear the value of the current row, and the applied filter
     * @param {MouseEvent} event 
     * @param {Boolean} [preventUpdate] wheter or not the clear function should reload the data 
     */
    function clear(event, preventUpdate) {
        if (activeRequests)
            cancelRequest();
        $scope.fieldRow = null;
        // labelElement.classList.remove('rui-txt-input-label-val');
        inputElement.value = "";
        clearButtonElement.style.display = 'none';
        // if (!preventUpdate) {
        //     update();
        // }
        if (!cacheBase.hasOwnProperty('')) {
            update();
        }
        else {
            $scope.field.dataSource.data = cacheBase[""];
        }
        destroyOutData();
    }

    // /**
    //  * 
    //  * @param {Object} rowObject -- a ZhRowObject to extract the data and insert on the currentRow
    //  */
    // function insertOutData(rowObject) {
    //     for (var key in $scope.field.outData) {
    //         var value = rowObject[key];
    //         $scope.field.widget.currentRow[$scope.field.outData[key]] = value;
    //     }
    // }

    // function destroyOutData() {
    //     for (var key in $scope.field.outData) {
    //         delete $scope.field.widget.currentRow[$scope.field.outData[key]];
    //     }
    // }

    async function fetchData() {
        //console.log('%cFetch ' + filterConditions.value, 'color: blue;');
        var deferredToCancelRequest = $q.defer();
        deferredToCancelRequest.reqText = inputElement.value;
        activeRequests.push(deferredToCancelRequest);
        spinner.classList.remove('hidden');
        // preventZhLoaderFromShowing();
        ScreenService.hideLoader();
        var dataSourceFilter = $scope.field.dataSourceFilter || 
                               $scope.field.dataSource.dataSourceFilter || [];
        var newFilter = dataSourceFilter.slice(0); // make a copy of the zhDataSourceFilter to keep it pristine
        newFilter.push({
            name: filterConditions.name,
            value: "%" + inputElement.value + "%",
            operator: "LIKE_ALL"
        }); // append our custom conditions into the new array
        var promisseToReturn = await $scope.field.dataSource.load(
            newFilter,
            null,
            null,
            () => {},
            () => {}, -1,
            deferredToCancelRequest.promise,
            () => {}
        ).then(function(resultSet) {
            if (resultSet) {
                doCache(deferredToCancelRequest.reqText, 
                    filteredAditionalOptions.concat(resultSet.dataset[Object.keys(resultSet.dataset)[0]])
                );
            }
            else {
                // console.log('canceled');
                ZhLogger.setLevel(ZhLogger.previousLevel);
            }
            deferredToCancelRequest = false;
        }, function(errorArg) {}
        ).finally(function() {
            hideSpinner();
        });
        // ScreenService.hideLoader();
        return promisseToReturn;
    }

    let restoreHeaderZIndex = () => {
        let header = document.querySelector('.mymenu-header');
        header.style.zIndex = 2;
        window.removeEventListener('popstate', restoreHeaderZIndex);
    };

    $scope.openPopUp = () => {
        let header = document.querySelector('.mymenu-header');
        header.style.zIndex = 'unset'
        window.addEventListener('popstate', restoreHeaderZIndex, false);
        popUp.classList.add('visible');
        // $scope.hideHeaderActions();
        if(!$scope.field.preventAutoFocus){
            inputElement.focus();
        }
        ApplicationContext.UIManager.pushState(() => {
            $scope.closePopUp(); 
            // $scope.restoreHeaderActions();
        });

        zhLoader.classList.add('invisible');
    };
    
    $scope.closePopUp = () => {
        popUp.classList.remove('visible');
        zhLoader.classList.remove('invisible');
        let header = document.querySelector('.mymenu-header');
        header.style.zIndex = 2;
        window.removeEventListener('popstate', restoreHeaderZIndex);
    }

    // @deprecated
    /*
    let hideSelectAutoCompleteAction = {
        "name": "cancelEdit",
        "label": "cancel",
        "ruiIcon": "fa-arrow-left",
        "isVisible": true,
        "showAsAction": "back",
        "events": [{
            "name": "ActionEvent",
            "code": "UIManager.goBack();"
        }]
    };

    $scope.hideHeaderActions = () => {
        $scope.headerStateBackup = {
            headerActions: $scope.widget.headerActions,
            showMenu: $scope.widget.showMenu,
            sortable: $scope.widget.sortable,
            showFilter: $scope.widget.showFilter,
            label: $scope.widget.container.label
        };
        $scope.widget.headerActions = [angular.copy(hideSelectAutoCompleteAction)];
        $scope.widget.showMenu = false;
        $scope.widget.sortable = false;
        $scope.widget.showFilter = false;
        $scope.widget.container.label = $scope.field.label;
    };

    $scope.restoreHeaderActions = () => {
        $scope.widget.container.label = $scope.headerStateBackup.label;
        delete $scope.headerStateBackup.label;

        for (const backupKeyName in $scope.headerStateBackup) {
            $scope.widget[backupKeyName] = $scope.headerStateBackup[backupKeyName];
            delete $scope.headerStateBackup[backupKeyName];
        }
    };
    */
}
// jshint ignore: end

// FILE: ./assets/templates/controllers/Utilities.js
function Utilities(ScreenService) {
    
    this.translate = function (e) {
        localStorage.setItem('language', e.target.name);
        this.selectFlag(e);
        ScreenService.setLanguage(e.target.name);
    };

    this.removeSelectedFlag = function() {
        document.getElementsByClassName('flag-selected')[0].classList.add("flag");
        document.getElementsByClassName('flag-selected')[0].classList.remove("flag-selected");
    };

    this.selectFlag = function(e) {
        this.removeSelectedFlag();
        e.target.classList.add("flag-selected");
        e.target.classList.remove("flag");
    };

    this.onInit = function() {
        const language = JSON.parse(localStorage.getItem('language')) || 'pt_br';

        document.getElementsByClassName('flag-selected')[0].classList.add("flag");
        document.getElementsByClassName('flag-selected')[0].classList.remove("flag-selected");

        document.getElementsByName(language)[0].classList.add("flag-selected");
        document.getElementsByName(language)[0].classList.remove("flag");
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register('Utilities', Utilities);
});

// FILE: ./assets/templates/header/header.js
function mymenuHeaderController($scope, templateManager, ScreenService) {

    var self = this;

    this.getTitle = function () {
        var container = templateManager.container;
        return container.label;
    };

    this.goBack = function () {
        let acWidget = templateManager.container.getActiveWidget();
        if (acWidget.name === 'qrCodeWidget' && acWidget.isScaning === true) {
            acWidget.isScaning = false;
            document.body.style.background = '';
            if ('QRScanner' in window) {
                QRScanner.getStatus((status) => {
                    if (status.scanning && status.showing) {
                        QRScanner.destroy();
                    }
                });
            }
        } else {
            window.history.back();
        }
    };

    this.menuActionVisibility = function () {
        var container = templateManager.container;
        return container && container.showMenu;
    };

    this.getActionsByType = function (type) {
        var container = templateManager.container;
        var widget = container.getActiveWidget();
        var actions = widget.actions || [];
        return actions.filter(function (action) {
            return action.showAsAction == type;
        });
    };

    this.toggleHeaderDropdown = function () {
        $scope.headerDropdownOpened = !$scope.headerDropdownOpened;
    };
}

// FILE: ./assets/templates/list-pesquisa/list.js
function MyMenuListController($scope, $rootScope, ZHPromise, localStorageEngine, templateManager, $sce) {

	var previousScroll;
	var container = $("#mym-list-container");
	var self = this;
	var defaultListProperties = {
		showImage: false,
		clearChangedOnReload: true
	};
	$scope.listProperties = $scope.widget.listProperties || defaultListProperties;
	$scope.rows = $scope.widget.dataSource.data;
	$scope.widget.itemsPerPage = $scope.widget.itemsPerPage || $scope.widget.dataSource.itemsPerPage;
    $scope.loadedAllData = false;
    $scope.loadingData = false;
	$scope.showLoadMore = true;
	var overlay = document.querySelector(".mym-list-overlay");

	container.nonbounce();

	this.getContainer = function() {
		return templateManager.container;
	};

	this.getListType = function(widget){
		var listType = "default";
		if (widget.listProperties) {
			listType = widget.listProperties.listType || listType;
		}
		return listType;
	};

	$scope.filterTitle = function (item) {
		if (item.isVisible && item.showOnList && item.showAsTitle) {
			return item;
		}
	};

	$scope.filterSubtitle = function (item) {
		if (item.isVisible && item.showOnList && !item.showAsTitle) {
			return item;
		}
	};

	$scope.noMatchVisibility = function() {
		var widget = self.getActiveWidget();
		return Util.isUndefinedOrNull(widget.searchCriteria) || !Object.keys(widget.searchCriteria).length;
	};

	$scope.searchNoMatchVisibility = function() {
		var widget = self.getActiveWidget();
		return !Util.isUndefinedOrNull(widget.searchCriteria) && Object.keys(widget.searchCriteria).length;
	};

	this.onLongTapRow = function (row) {
		var dataArray = $scope.widget.dataSource.data.filter(function(row) {
			return row.dropdownOpened;
		});
		if (!row.__isSelected && $scope.widget.allowSelectOnLongTap && dataArray.length === 0) {
			var widget = self.getActiveWidget();
			for(var i = 0; i< dataArray.length; i++) {
				dataArray[i].dropdownOpened = false;
				document.getElementById('dropdown_' + dataArray[i].$$hashKey).parentElement.classList.remove('list-dropdown-opened');
			}
			$scope.widget.selectAll([row]);
			if (window.navigator.vibrate) {
				window.navigator.vibrate(25);
			}
		}
	};

	this.closeDropdown = function(row) {
		event.preventDefault();
		event.stopPropagation();
		var dataArray = $scope.widget.dataSource.data.filter(function(row) {
			return row.dropdownOpened;
		});
		if(dataArray.length > 0) {
			for(var i = 0; i < dataArray.length; i++) {
				dataArray[i].dropdownOpened = false;
				document.getElementById('dropdown_'+dataArray[i].$$hashKey).parentElement.classList.remove('list-dropdown-opened');
			}
		} else {
			this.setCurrentRow(row);
		}
		overlay.style.display = "none";
	};

	this.setCurrentRow = function (row) {		
		var widget = $scope.widget;		
		if (widget.beforeMoveRow) {		
			event = widget.beforeMoveRow();		
		}		
		if (!event || !event.defaultPrevented) {		
			widget.currentRow = row;		
			if (widget.currentRow && widget.dataSource && widget.dataSource.data) {		
				widget.currentRow.__index = widget.dataSource.data.indexOf(row);		
			}		
			if (templateManager.project.showDescriptionOrder) {		
				widget.computeDescriptionOrder();		
			}		
		}		
		if (!event.cancelBubble) {		
			if (widget.afterMoveRow) {		
				widget.afterMoveRow();		
			}		
		}		
	};

	this.getCheckboxIcon = function(row) {
		if (row.__isSelected) {
			return "checkbox";
		} else {
			return "checkboxoutlineblank";
		}
	};
	this.changeColorField = function(field,widget) {
		var classes;
		var currStatus = field.IDSTATUS;
		var widgetfields = widget.fields;

		widgetfields.forEach(function(fieldOfWidget){
			if(fieldOfWidget.name === "IDSTATUS"){
				classes = fieldOfWidget.classesForTitle;	
			}
		});
		
		if(Util.isUndefinedOrNull(classes)){
			return null;
		}
		if(Util.isUndefinedOrNull(classes[currStatus])){
			return classes.APROVADO;	
		}
		return classes[currStatus];
	};
	this.filteredRows = function() {
		var widget = $scope.widget;
		var searchCriteria = widget.searchCriteria || {};
		var data  = widget.dataSource.data || [];

		function filterComparising(row) {
			var isVisible = false;
			if (Util.objectHasAnyValue(searchCriteria)) {
				Object.keys(searchCriteria).forEach(function(filter) {
					if (!isVisible) {
						if (filter === "_ALL") {
							var fields;
							if (widget.headerSearchBy) {
								widget.headerSearchBy = Array.isArray(widget.headerSearchBy) ? widget.headerSearchBy : [widget.headerSearchBy];
								fields = widget.headerSearchBy;
							} else {
								fields = getVisibleFields();
							}
							fields.forEach(function(field) {
								if (!isVisible) {
									isVisible = compareValues(row[field], searchCriteria[filter], isVisible);
								}
							});
						} else {
							isVisible = compareValues(row[filter], searchCriteria[filter], isVisible);
						}
					}
				});
			} else {
				isVisible = true;
			}
			return isVisible;
		}

		function getVisibleFields() {
			return widget.fields.map(function(field) {
				if (field.isVisible && field.showOnList) {
					return field.name;
				}
			});
		}

		function compareValues(rowValue, filterValue) {
			if (!Util.isUndefinedOrNull(rowValue)) {
				rowValue = rowValue.toLowerCase();
				rowValue = rowValue.latinize();
				filterValue = filterValue.toLowerCase();
				filterValue = filterValue.latinize();
				if (rowValue.search(filterValue) !== -1) {
					return true;
				}
			}
			return false;
		}
		return data.filter(filterComparising);
	};

	this.toggleImageCheckbox = function(row) {
		var widget = self.getActiveWidget();
		if (widget.listProperties.showCheckbox) {
			this.toggle(row);
		}
	};


	this.openDetail = function (row, widget) {
		row.__showDetail = true;
		widget.detailRow = row;
		widget.isShowingDetail = true;
		if (widget.detailWidget) {
			var widgetToActivate = widget.container.getWidget(widget.detailWidget);
			widgetToActivate.listDetailWidget = widget;
			widgetToActivate.activate();
		}
	};

	this.closeDetail = function (widget) {
		var row = widget.detailRow;
		row.__showDetail = false;
		delete widget.detailRow;
		widget.isShowingDetail = false;
		if (widget.listDetailWidget) {
			var widgetToActivate = widget.listDetailWidget;
			widgetToActivate.activate();
		}
	};
	

	this.getActiveWidget = function() {
		return self.getContainer().getActiveWidget();
	};

	this.getClosedStatus = function(row) {
		var status = false;
		if (row && row.__groupHeaders && row.__groupHeaders.length > 0) {
			status = row.__groupHeaders.some(function(group) {
				return group.__groupOpened === false;
			});
		}
		return status;
	};

	this.toggle = function(row) {
		var widget = $scope.widget;
		widget.selectedRow = row;

		var event;
		if (widget.beforeSelectRow && !row.__isSelected) {
			event = widget.beforeSelectRow();
		} else if (widget.beforeUnselectRow && row.__isSelected) {
			event = widget.beforeUnselectRow();
		}
		if (!event || !event.isDefaultPrevented()) {
			var index = getIndex(row);
			if (index === -1) {
				row.__isSelected = true;
				if (!$scope._currentPopup) $rootScope.$broadcast("ui.addToFooter", "showOnMultiSelect");
			} else {
				delete row.__isSelected;
				widget.dataSource.removeCheckedRows(row);
			}
			if (!$scope._currentPopup && !widget.dataSource.checkedRows.length) {
				$rootScope.$broadcast("ui.removeFromFooter", "showOnMultiSelect");
			}

			widget.updateCheckedRows();

			if (widget.afterSelectRow && row.__isSelected) {
				widget.afterSelectRow();
			} else if (widget.afterUnselectRow && !row.__isSelected) {
				widget.afterUnselectRow();
			}
		}
	};

	function getIndex(row) {
		return $scope.widget.dataSource.checkedRows.indexOf(row);
	}

	this.toggleDropdown = function(row, e) {
		if (!$scope.widget.hasCheckedRows()) {
			var dataArray = $scope.widget.dataSource.data.filter(function(row) {
				return row.dropdownOpened === true;
			});

			if(dataArray.length < 1) {
				var htmlElementTarget = e.target;
				var dropdownContent = htmlElementTarget.parentElement.parentElement.parentElement.lastElementChild.firstElementChild;
				var dropdown = htmlElementTarget.parentElement.parentElement.parentElement.lastElementChild;
				var availSpace = screen.availHeight - dropdownContent.getBoundingClientRect().top - (screen.availHeight/15);
				if (availSpace > dropdownContent.offsetHeight) {
					dropdownContent.classList.remove('list-dropdown-last');
				} else {
					dropdownContent.classList.add('list-dropdown-last');
				}
				if(!($scope.widget.hasCheckedRows())) {
					row.dropdownOpened = !row.dropdownOpened;
					if(row.dropdownOpened) {
						dropdown.classList.add('list-dropdown-opened');
					} else {
						dropdown.classList.remove('list-dropdown-opened');
					}
				}
				overlay.style.display = "block";
			} else {
				for(var i=0; i<$scope.widget.dataSource.data.length; i++) {
					$scope.widget.dataSource.data[i].dropdownOpened=false;
					document.getElementById('dropdown_'+i).parentElement.classList.remove('list-dropdown-opened');
				}
			}
			e.preventDefault();
			e.stopPropagation();
		} else {
			e.stopPropagation();
			self.toggle(row);
		}
	};

	this.onActionClick = function(action, actionClick, row) {
		if(!($scope.widget.hasCheckedRows())) {
			var activeWidget = self.getActiveWidget();
			event.stopPropagation();
			event.preventDefault();
			actionClick(action, activeWidget, row);
		} else {
			self.toggle(row);
		}
		self.closeDropdown(row);
	};

   	/**
     * Provide endless scroll on list container
     */
    this.infiniteScroll = function() {
        var scrollHeight = container[0].scrollHeight;
        var scrollPosition = container.height() + container.scrollTop();
        previousScroll = previousScroll || scrollPosition;
        if (((scrollHeight - scrollPosition) / scrollHeight < 0.030) && previousScroll < scrollPosition && !$scope.loadingData && !$scope.loadedAllData) {
            this.loadData(true);
        }
        previousScroll = scrollPosition;
    };

    /**
     * Retrieve content from list dataSource
     * @param  {Boolean} apply - Scope apply selector
     */
    this.loadData = function(apply) {
        $scope.loadingData = true;
    	$('.default-window>.zh-background-loading').addClass("list-hide-loader-overlay");
    	$('.zh-background-loading>.zh-position-loading>.zh-loading-icon').css("display", "none");
        if (apply) {
            $scope.$apply();
        }
        var widget = $scope.widget;
        this.nextPage(widget).then(function(callback) {
        	var data;
            if (callback && callback.dataset) {
                Object.keys(callback.dataset).forEach(function (datasetName) {
                    data = callback.dataset[datasetName];
                });
            } else {
                data = callback || [];
            }
            $scope.loadedAllData = data.length < widget.itemsPerPage;
            this.loadMoreVisibility();
            $scope.loadingData = false;
            $('.default-window>.zh-background-loading').removeClass("list-hide-loader-overlay");
	        $('.zh-background-loading>.zh-position-loading>.zh-loading-icon').css("display", "inline-block");
        }.bind(this));
    };

    /**
     * Manage load more button visibility
     */
    this.loadMoreVisibility = function() {
    	var listContainer = $('#mym-list-container')[0];
    	if (listContainer) {
	        var scrollHeight = listContainer.scrollHeight + $('.lf-navbar').height();
	        var windowHeight = $(window).height();
	        if (scrollHeight > windowHeight) {
	            $scope.showLoadMore = false;
	        } else {
	            $scope.showLoadMore = true;
	        }

    	}
    };

    /**
     * Retrieve next page of widget's datasource
     * @param  {Widget} widget
     * @return {ZHPromise} Load function callback
     */
    this.nextPage = function(widget) {
        if (widget.dataSource) {
            return widget.dataSource.load(widget.getDataSourceFilter(), widget.dataSource.getNextPage(), widget.itemsPerPage);
        } else {
            return ZHPromise.when();
        }
    };

    this.getRowImage = function(row) {
    	var title = "";
    	if ($scope.listProperties.imageTitleField && row[$scope.listProperties.imageTitleField]) {
    		title = row[$scope.listProperties.imageTitleField].toString().substr(0,1);
    	}
    	return title;
    };

    this.isActionVisible = function(row, action) {
    	var isVisible = action.condition ? eval(action.condition) : action.isVisible; // jshint ignore:line
    	return isVisible;
	};
	
	this.trustAsHtml = function(htmlContent) {
		return $sce.trustAsHtml(htmlContent);
	};

    $scope.filterRowActions = function(action) {
    	if (action.showAsAction.indexOf("|") === -1) {
    		return action.showAsAction === "row";
    	} else {
    		var showAsAction = action.showAsAction.split("|");
    		var isVisible = false;
    		for (var i = 0; i < showAsAction.length; i++) {
    			if (showAsAction[i] === "row") {
    				isVisible = true;
    				break;
    			}
    		}
    		return isVisible;
    	}
    };
	/**
	 * Overwrite default reload action of widget
	 * @param {Object[]} additionalFilters - (Optional) Filters to be applied on reload
	 * @param {Boolean} shouldRefreshCurrentRow - (Optional) True if should refresh current row
	 */
	if (!$scope.widget.defaultReload) {
		$scope.widget.defaultReload = angular.copy($scope.widget.reload);
	    $scope.widget.reload = function(additionalFilters, shouldRefreshCurrentRow) {
	    	$scope.loadingData = true;
	    	$('.default-window>.zh-background-loading').addClass("list-hide-loader-overlay");
    		$('.zh-background-loading>.zh-position-loading>.zh-loading-icon').css("display", "none");
	        if ($scope.listProperties.clearChangedOnReload) {
	        	$scope.widget.dataSource.clearChangedRows();
	        }
	        return $scope.widget.defaultReload(additionalFilters, shouldRefreshCurrentRow).then(function() {
	            container.scrollTop(0);
	            $scope.loadedAllData = false;
	            $scope.loadingData = false;
	            $('.default-window>.zh-background-loading').removeClass("list-hide-loader-overlay");
	        	$('.zh-background-loading>.zh-position-loading>.zh-loading-icon').css("display", "inline-block");
	            $scope.widget.dataSource.updateChangedRows();
	        }.bind(this));
	    };
	}

	if (!$scope.widget.dataSource.updateChangedRows) {
		var dataSource = $scope.widget.dataSource;
		var ignoreKeys = ['__rightActions', '__leftActions'];

		dataSource._changedRows = [];
		dataSource._changedRowsCache = {};

		dataSource.getHashKey = function(row) {
			var VO = this.getVO();
			var hash = "";
			if (VO && VO.localKey) {
				hash = VO.localKey.reduce(function(hash, key) {
					return row[key] ? hash + "_" + row[key].toString() : hash;
				}, "");
			}
			return hash;
		};

		dataSource.addChangedRows = function (rows) {
			rows = Util.parseToArray(rows);
			rows.forEach(function(row) {
				if (this._changedRows.indexOf(row) === -1) {
					this._changedRows.push(row);
				}
				var hash = this.getHashKey(row);
				if (hash) {
					this._changedRowsCache[hash] = Util.clone(row, ignoreKeys);
				}
			}.bind(this));
		};

		dataSource.clearChangedRows = function() {
			this._changedRows = [];
			this._changedRowsCache = {};
		};

		dataSource.getChangedRows = function() {
			var changedRows = this._changedRows;
			if (this.name) {
				var changedRowsHash = changedRows.map(function(row) {
					return this.getHashKey(row);
				}.bind(this));
				changedRows = changedRows.concat(Object.keys(this._changedRowsCache).filter(function(row) {
					return changedRowsHash.indexOf(row) === -1;
				}.bind(this)).map(function(hash) {
					return this._changedRowsCache[hash];
				}.bind(this)));
			}
			return changedRows;
		};

		dataSource.updateChangedRows = function() {
			if (this.data) {
				this._changedRows = this.data.filter(function(row) {
					var hash = this.getHashKey(row);
					if (this._changedRowsCache.hasOwnProperty(hash)) {
						for (var i in this._changedRowsCache[hash]) {
							row[i] = this._changedRowsCache[hash][i];
						}
					}
					return row.changed;
				}.bind(this));
			} else {
				this.clearChangedRows();
			}
			if (templateManager.container) {
				templateManager.container.updateCheckedRows();
			}
		};
	}

	if (!$scope.widget.cancel) {
		$scope.widget.cancel = this.cancel;
	}

    container.on("scroll", this.infiniteScroll.bind(this));
    $(window).on("resize", this.loadMoreVisibility);
}

// FILE: ./assets/templates/list/list.js
function MyMenuListController($scope, $rootScope, ZHPromise, localStorageEngine, templateManager, $sce, ScreenService) {

	var previousScroll;
	var container = $("#mym-list-container");
	var self = this;
	var defaultListProperties = {
		showImage: false,
		clearChangedOnReload: true
	};
	$scope.listProperties = $scope.widget.listProperties || defaultListProperties;
	$scope.rows = $scope.widget.dataSource.data;
	$scope.widget.itemsPerPage = $scope.widget.itemsPerPage || $scope.widget.dataSource.itemsPerPage;
	$scope.loadedAllData = false;
	$scope.loadingData = false;
	$scope.showLoadMore = true;
	var overlay = document.querySelector(".mym-list-overlay");

	container.nonbounce();

	this.getContainer = function () {
		return templateManager.container;
	};

	this.getListType = function (widget) {
		var listType = "default";
		if (widget.listProperties) {
			listType = widget.listProperties.listType || listType;
		}
		return listType;
	};

	$scope.filterTitle = function (item) {
		if (item.isVisible && item.showOnList && item.showAsTitle) {
			return item;
		}
	};

	$scope.filterSubtitle = function (item) {
		if (item.isVisible && item.showOnList && !item.showAsTitle) {
			return item;
		}
	};

	$scope.noMatchVisibility = function () {
		var widget = self.getActiveWidget();
		return Util.isUndefinedOrNull(widget.searchCriteria) || !Object.keys(widget.searchCriteria).length;
	};

	$scope.searchNoMatchVisibility = function () {
		var widget = self.getActiveWidget();
		return !Util.isUndefinedOrNull(widget.searchCriteria) && Object.keys(widget.searchCriteria).length;
	};

	this.onLongTapRow = function (row) {
		var dataArray = $scope.widget.dataSource.data.filter(function (row) {
			return row.dropdownOpened;
		});
		if (!row.__isSelected && $scope.widget.allowSelectOnLongTap && dataArray.length === 0) {
			var widget = self.getActiveWidget();
			for (var i = 0; i < dataArray.length; i++) {
				dataArray[i].dropdownOpened = false;
				document.getElementById('dropdown_' + dataArray[i].$$hashKey).parentElement.classList.remove('list-dropdown-opened');
			}
			$scope.widget.selectAll([row]);
			if (window.navigator.vibrate) {
				window.navigator.vibrate(25);
			}
		}
	};

	this.closeDropdown = function (row) {
		event.preventDefault();
		event.stopPropagation();
		var dataArray = $scope.widget.dataSource.data.filter(function (row) {
			return row.dropdownOpened;
		});
		if (dataArray.length > 0) {
			for (var i = 0; i < dataArray.length; i++) {
				dataArray[i].dropdownOpened = false;
				document.getElementById('dropdown_' + dataArray[i].$$hashKey).parentElement.classList.remove('list-dropdown-opened');
			}
		} else {
			this.setCurrentRow(row);
		}
		overlay.style.display = "none";
	};

	this.setCurrentRow = function (row) {
		var widget = $scope.widget;
		if (widget.beforeMoveRow) {
			event = widget.beforeMoveRow();
		}
		if (!event || !event.defaultPrevented) {
			widget.currentRow = row;
			if (widget.currentRow && widget.dataSource && widget.dataSource.data) {
				widget.currentRow.__index = widget.dataSource.data.indexOf(row);
			}
			if (templateManager.project.showDescriptionOrder) {
				widget.computeDescriptionOrder();
			}
		}
		if (!event.cancelBubble) {
			if (widget.afterMoveRow) {
				widget.afterMoveRow();
			}
		}
	};

	this.getCheckboxIcon = function (row) {
		if (row.__isSelected) {
			return "checkbox";
		} else {
			return "checkboxoutlineblank";
		}
	};
	this.changeColorField = function (field, widget) {
		var classes;
		var currStatus = field.IDSTATUS;
		var widgetfields = widget.fields;

		widgetfields.forEach(function (fieldOfWidget) {
			if (fieldOfWidget.name === "IDSTATUS") {
				classes = fieldOfWidget.classesForTitle;
			}
		});

		if (Util.isUndefinedOrNull(classes)) {
			return null;
		}
		if (Util.isUndefinedOrNull(classes[currStatus])) {
			return classes.APROVADO;
		}
		return classes[currStatus];
	};
	this.filteredRows = function () {
		var widget = $scope.widget;
		var searchCriteria = widget.searchCriteria || {};
		var data = widget.dataSource.data || [];

		function filterComparising(row) {
			var isVisible = false;
			if (Util.objectHasAnyValue(searchCriteria)) {
				Object.keys(searchCriteria).forEach(function (filter) {
					if (!isVisible) {
						if (filter === "_ALL") {
							var fields;
							if (widget.headerSearchBy) {
								widget.headerSearchBy = Array.isArray(widget.headerSearchBy) ? widget.headerSearchBy : [widget.headerSearchBy];
								fields = widget.headerSearchBy;
							} else {
								fields = getVisibleFields();
							}
							fields.forEach(function (field) {
								if (!isVisible) {
									isVisible = compareValues(row[field], searchCriteria[filter], isVisible);
								}
							});
						} else {
							isVisible = compareValues(row[filter], searchCriteria[filter], isVisible);
						}
					}
				});
			} else {
				isVisible = true;
			}
			return isVisible;
		}

		function getVisibleFields() {
			return widget.fields.map(function (field) {
				if (field.isVisible && field.showOnList) {
					return field.name;
				}
			});
		}

		function compareValues(rowValue, filterValue) {
			if (!Util.isUndefinedOrNull(rowValue)) {
				rowValue = rowValue.toLowerCase();
				rowValue = rowValue.latinize();
				filterValue = filterValue.toLowerCase();
				filterValue = filterValue.latinize();
				if (rowValue.search(filterValue) !== -1) {
					return true;
				}
			}
			return false;
		}
		return data.filter(filterComparising);
	};

	this.toggleImageCheckbox = function (row) {
		var widget = self.getActiveWidget();
		if (widget.listProperties.showCheckbox) {
			this.toggle(row);
		}
	};


	this.openDetail = function (row, widget) {
		widget.activate();
		row.__showDetail = true;
		widget.detailRow = row;
		widget.isShowingDetail = true;
		if (widget.detailWidget) {
			var detailWidget = widget.container.getWidget(widget.detailWidget);
			detailWidget.listDetailWidget = widget.name;
			widget.__detailWidget = detailWidget;
		}
		if (row && row.NMPRATO) {
			widget.container.label = row.NMPRATO;
			let backAction = widget.container.getWidget('mainWidget').actions[0];
			backAction.isVisible = backAction.isVisible? backAction.isVisible : true;
		}
	};

	// this.closeDetail = function (widget) {
	// 	var row = widget.detailRow;
	// 	row.__showDetail = false;
	// 	delete widget.detailRow;
	// 	widget.isShowingDetail = false;

	// 	if (widget.listDetailWidget) {
	// 		var listDetailWidget = widget.container.getWidget(widget.listDetailWidget);
	// 		listDetailWidget.activate();
	// 		widget.container.label = listDetailWidget.detailRow.NMPRATO;
	// 	}else{
	// 		widget.container.label = ScreenService.i18n('Cardápio');
	// 	}
	// };


	this.getActiveWidget = function () {
		return self.getContainer().getActiveWidget();
	};

	this.getClosedStatus = function (row) {
		var status = false;
		if (row && row.__groupHeaders && row.__groupHeaders.length > 0) {
			status = row.__groupHeaders.some(function (group) {
				return group.__groupOpened === false;
			});
		}
		return status;
	};

	this.toggle = function (row) {
		var widget = $scope.widget;
		widget.selectedRow = row;

		var event;
		if (widget.beforeSelectRow && !row.__isSelected) {
			event = widget.beforeSelectRow();
		} else if (widget.beforeUnselectRow && row.__isSelected) {
			event = widget.beforeUnselectRow();
		}
		if (!event || !event.isDefaultPrevented()) {
			var index = getIndex(row);
			if (index === -1) {
				row.__isSelected = true;
				if (!$scope._currentPopup) $rootScope.$broadcast("ui.addToFooter", "showOnMultiSelect");
			} else {
				delete row.__isSelected;
				widget.dataSource.removeCheckedRows(row);
			}
			if (!$scope._currentPopup && !widget.dataSource.checkedRows.length) {
				$rootScope.$broadcast("ui.removeFromFooter", "showOnMultiSelect");
			}

			widget.updateCheckedRows();

			if (widget.afterSelectRow && row.__isSelected) {
				widget.afterSelectRow();
			} else if (widget.afterUnselectRow && !row.__isSelected) {
				widget.afterUnselectRow();
			}
		}
	};

	function getIndex(row) {
		return $scope.widget.dataSource.checkedRows.indexOf(row);
	}

	this.toggleDropdown = function (row, e) {
		if (!$scope.widget.hasCheckedRows()) {
			var dataArray = $scope.widget.dataSource.data.filter(function (row) {
				return row.dropdownOpened === true;
			});

			if (dataArray.length < 1) {
				var htmlElementTarget = e.target;
				var dropdownContent = htmlElementTarget.parentElement.parentElement.parentElement.lastElementChild.firstElementChild;
				var dropdown = htmlElementTarget.parentElement.parentElement.parentElement.lastElementChild;
				var availSpace = screen.availHeight - dropdownContent.getBoundingClientRect().top - (screen.availHeight / 15);
				if (availSpace > dropdownContent.offsetHeight) {
					dropdownContent.classList.remove('list-dropdown-last');
				} else {
					dropdownContent.classList.add('list-dropdown-last');
				}
				if (!($scope.widget.hasCheckedRows())) {
					row.dropdownOpened = !row.dropdownOpened;
					if (row.dropdownOpened) {
						dropdown.classList.add('list-dropdown-opened');
					} else {
						dropdown.classList.remove('list-dropdown-opened');
					}
				}
				overlay.style.display = "block";
			} else {
				for (var i = 0; i < $scope.widget.dataSource.data.length; i++) {
					$scope.widget.dataSource.data[i].dropdownOpened = false;
					document.getElementById('dropdown_' + i).parentElement.classList.remove('list-dropdown-opened');
				}
			}
			e.preventDefault();
			e.stopPropagation();
		} else {
			e.stopPropagation();
			self.toggle(row);
		}
	};

	this.onActionClick = function (action, actionClick, row) {
		if (!($scope.widget.hasCheckedRows())) {
			var activeWidget = self.getActiveWidget();
			event.stopPropagation();
			event.preventDefault();
			actionClick(action, activeWidget, row);
		} else {
			self.toggle(row);
		}
		self.closeDropdown(row);
	};

	/**
	 * Provide endless scroll on list container
	 */
	this.infiniteScroll = function () {
		var scrollHeight = container[0].scrollHeight;
		var scrollPosition = container.height() + container.scrollTop();
		previousScroll = previousScroll || scrollPosition;
		if (((scrollHeight - scrollPosition) / scrollHeight < 0.030) && previousScroll < scrollPosition && !$scope.loadingData && !$scope.loadedAllData) {
			this.loadData(true);
		}
		previousScroll = scrollPosition;
	};

	/**
	 * Retrieve content from list dataSource
	 * @param  {Boolean} apply - Scope apply selector
	 */
	this.loadData = function (apply) {
		$scope.loadingData = true;
		$('.default-window>.zh-background-loading').addClass("list-hide-loader-overlay");
		$('.zh-background-loading>.zh-position-loading>.zh-loading-icon').css("display", "none");
		if (apply) {
			$scope.$apply();
		}
		var widget = $scope.widget;
		this.nextPage(widget).then(function (callback) {
			var data;
			if (callback && callback.dataset) {
				Object.keys(callback.dataset).forEach(function (datasetName) {
					data = callback.dataset[datasetName];
				});
			} else {
				data = callback || [];
			}
			$scope.loadedAllData = data.length < widget.itemsPerPage;
			this.loadMoreVisibility();
			$scope.loadingData = false;
			$('.default-window>.zh-background-loading').removeClass("list-hide-loader-overlay");
			$('.zh-background-loading>.zh-position-loading>.zh-loading-icon').css("display", "inline-block");
		}.bind(this));
	};

	/**
	 * Manage load more button visibility
	 */
	this.loadMoreVisibility = function () {
		var listContainer = $('#mym-list-container')[0];
		if (listContainer) {
			var scrollHeight = listContainer.scrollHeight + $('.lf-navbar').height();
			var windowHeight = $(window).height();
			if (scrollHeight > windowHeight) {
				$scope.showLoadMore = false;
			} else {
				$scope.showLoadMore = true;
			}

		}
	};

	/**
	 * Retrieve next page of widget's datasource
	 * @param  {Widget} widget
	 * @return {ZHPromise} Load function callback
	 */
	this.nextPage = function (widget) {
		if (widget.dataSource) {
			return widget.dataSource.load(widget.getDataSourceFilter(), widget.dataSource.getNextPage(), widget.itemsPerPage);
		} else {
			return ZHPromise.when();
		}
	};

	this.getRowImage = function (row) {
		var title = "";
		if ($scope.listProperties.imageTitleField && row[$scope.listProperties.imageTitleField]) {
			title = row[$scope.listProperties.imageTitleField].toString().substr(0, 1);
		}
		return title;
	};

	this.isActionVisible = function (row, action) {
		var isVisible = action.condition ? eval(action.condition) : action.isVisible; // jshint ignore:line
		return isVisible;
	};

	this.trustAsHtml = function (htmlContent) {
		return $sce.trustAsHtml(htmlContent);
	};

	$scope.filterRowActions = function (action) {
		if (action.showAsAction.indexOf("|") === -1) {
			return action.showAsAction === "row";
		} else {
			var showAsAction = action.showAsAction.split("|");
			var isVisible = false;
			for (var i = 0; i < showAsAction.length; i++) {
				if (showAsAction[i] === "row") {
					isVisible = true;
					break;
				}
			}
			return isVisible;
		}
	};
	/**
	 * Overwrite default reload action of widget
	 * @param {Object[]} additionalFilters - (Optional) Filters to be applied on reload
	 * @param {Boolean} shouldRefreshCurrentRow - (Optional) True if should refresh current row
	 */
	if (!$scope.widget.defaultReload) {
		$scope.widget.defaultReload = angular.copy($scope.widget.reload);
		$scope.widget.reload = function (additionalFilters, shouldRefreshCurrentRow) {
			$scope.loadingData = true;
			$('.default-window>.zh-background-loading').addClass("list-hide-loader-overlay");
			$('.zh-background-loading>.zh-position-loading>.zh-loading-icon').css("display", "none");
			if ($scope.listProperties.clearChangedOnReload) {
				$scope.widget.dataSource.clearChangedRows();
			}
			return $scope.widget.defaultReload(additionalFilters, shouldRefreshCurrentRow).then(function () {
				container.scrollTop(0);
				$scope.loadedAllData = false;
				$scope.loadingData = false;
				$('.default-window>.zh-background-loading').removeClass("list-hide-loader-overlay");
				$('.zh-background-loading>.zh-position-loading>.zh-loading-icon').css("display", "inline-block");
				$scope.widget.dataSource.updateChangedRows();
			}.bind(this));
		};
	}

	if (!$scope.widget.dataSource.updateChangedRows) {
		var dataSource = $scope.widget.dataSource;
		var ignoreKeys = ['__rightActions', '__leftActions'];

		dataSource._changedRows = [];
		dataSource._changedRowsCache = {};

		dataSource.getHashKey = function (row) {
			var VO = this.getVO();
			var hash = "";
			if (VO && VO.localKey) {
				hash = VO.localKey.reduce(function (hash, key) {
					return row[key] ? hash + "_" + row[key].toString() : hash;
				}, "");
			}
			return hash;
		};

		dataSource.addChangedRows = function (rows) {
			rows = Util.parseToArray(rows);
			rows.forEach(function (row) {
				if (this._changedRows.indexOf(row) === -1) {
					this._changedRows.push(row);
				}
				var hash = this.getHashKey(row);
				if (hash) {
					this._changedRowsCache[hash] = Util.clone(row, ignoreKeys);
				}
			}.bind(this));
		};

		dataSource.clearChangedRows = function () {
			this._changedRows = [];
			this._changedRowsCache = {};
		};

		dataSource.getChangedRows = function () {
			var changedRows = this._changedRows;
			if (this.name) {
				var changedRowsHash = changedRows.map(function (row) {
					return this.getHashKey(row);
				}.bind(this));
				changedRows = changedRows.concat(Object.keys(this._changedRowsCache).filter(function (row) {
					return changedRowsHash.indexOf(row) === -1;
				}.bind(this)).map(function (hash) {
					return this._changedRowsCache[hash];
				}.bind(this)));
			}
			return changedRows;
		};

		dataSource.updateChangedRows = function () {
			if (this.data) {
				this._changedRows = this.data.filter(function (row) {
					var hash = this.getHashKey(row);
					if (this._changedRowsCache.hasOwnProperty(hash)) {
						for (var i in this._changedRowsCache[hash]) {
							row[i] = this._changedRowsCache[hash][i];
						}
					}
					return row.changed;
				}.bind(this));
			} else {
				this.clearChangedRows();
			}
			if (templateManager.container) {
				templateManager.container.updateCheckedRows();
			}
		};
	}

	if (!$scope.widget.cancel) {
		$scope.widget.cancel = this.cancel;
	}

	container.on("scroll", this.infiniteScroll.bind(this));
	$(window).on("resize", this.loadMoreVisibility);
}

// FILE: ./assets/templates/simple-stripe/SimpleStripeController.js
function SimpleStripeController($scope) {

	/* DESIRED BEHAVIOR:
	*  - There is a limit of two visible actions on the widget
	*	 - Normal fields should show label and value
	*  - Normal fields should have their width customized by the developer
	*  - Direct inputs should have the same width
	*  - Direct inputs should occupy only one row
	*  - Direct inputs are not allowed on smartphones due to it's small screen nature
	*  - When collapsed the widget should show only the first two fields that have the flag 'showWhenCollapsed'
	*  - When there are no fields with 'showWhenCollapsed' the deafult behavior is to show the first two fields
	*/

    var self = this;

    this.__defineGetter__('widget', function () {
        return $scope.widget;
    });

    self.widget.directInput = self.widget.directInput && !Util.isPhone();
    $scope.fieldWidth = Math.max(1, parseInt(12 / self.widget.fields.length));

    function groupFieldsByRowSize(fields, property, maxSize) {
        if (fields) {
            var rows = [{
                fields: []
            }];
            var rowSize = 0;
            var colSize;
            for (var i = 0; i < fields.length; i++) {
                colSize = parseInt(fields[i][property], 10);
                if (colSize > maxSize) {
                    colSize = maxSize;
                    fields[i][property] = colSize;
                }

                rowSize += colSize;
                if (rowSize > maxSize || fields[i].breakline) {
                    rows.push({
                        fields: []
                    });
                    rowSize = colSize;
                }
                rows[rows.length - 1].fields.push(fields[i]);
            }
            return rows;
        }
    }

    function buildStripeFields(visibleFields) {
        var alwaysVisibleFields = visibleFields.filter(function (field) {
            return field.showWhenCollapsed;
        });

        var visibleRows = groupFieldsByRowSize(visibleFields, "class", 12);
        var alwaysVisibleRows = groupFieldsByRowSize(alwaysVisibleFields, "class", 12).slice(0, 2);

        if (alwaysVisibleFields.length === 0) {
            alwaysVisibleRows = visibleRows.slice(0, 2);
        }


        if (self.widget.collapsible) {

            $scope.collapsed = true;
            $scope.stripeRows = alwaysVisibleRows;
            $scope.shouldCollapse = visibleRows.length > 2;

            $scope.collapseRows = function () {
                $scope.collapsed = !$scope.collapsed;
                $scope.stripeRows = ($scope.collapsed) ? alwaysVisibleRows : visibleRows;
            };

        } else {
            $scope.stripeRows = visibleRows;
        }
    }

    $scope.$watchCollection(function () {
        return self.widget.fields.filter(function (field) {
            return field.isVisible && field.aclIsVisible;
        });
    }, function (fields) {
        buildStripeFields(fields);
    });
}
