function SimpleStripeController($scope) {

	/* DESIRED BEHAVIOR:
	*  - There is a limit of two visible actions on the widget
	*	 - Normal fields should show label and value
	*  - Normal fields should have their width customized by the developer
	*  - Direct inputs should have the same width
	*  - Direct inputs should occupy only one row
	*  - Direct inputs are not allowed on smartphones due to it's small screen nature
	*  - When collapsed the widget should show only the first two fields that have the flag 'showWhenCollapsed'
	*  - When there are no fields with 'showWhenCollapsed' the deafult behavior is to show the first two fields
	*/

    var self = this;

    this.__defineGetter__('widget', function () {
        return $scope.widget;
    });

    self.widget.directInput = self.widget.directInput && !Util.isPhone();
    $scope.fieldWidth = Math.max(1, parseInt(12 / self.widget.fields.length));

    function groupFieldsByRowSize(fields, property, maxSize) {
        if (fields) {
            var rows = [{
                fields: []
            }];
            var rowSize = 0;
            var colSize;
            for (var i = 0; i < fields.length; i++) {
                colSize = parseInt(fields[i][property], 10);
                if (colSize > maxSize) {
                    colSize = maxSize;
                    fields[i][property] = colSize;
                }

                rowSize += colSize;
                if (rowSize > maxSize || fields[i].breakline) {
                    rows.push({
                        fields: []
                    });
                    rowSize = colSize;
                }
                rows[rows.length - 1].fields.push(fields[i]);
            }
            return rows;
        }
    }

    function buildStripeFields(visibleFields) {
        var alwaysVisibleFields = visibleFields.filter(function (field) {
            return field.showWhenCollapsed;
        });

        var visibleRows = groupFieldsByRowSize(visibleFields, "class", 12);
        var alwaysVisibleRows = groupFieldsByRowSize(alwaysVisibleFields, "class", 12).slice(0, 2);

        if (alwaysVisibleFields.length === 0) {
            alwaysVisibleRows = visibleRows.slice(0, 2);
        }


        if (self.widget.collapsible) {

            $scope.collapsed = true;
            $scope.stripeRows = alwaysVisibleRows;
            $scope.shouldCollapse = visibleRows.length > 2;

            $scope.collapseRows = function () {
                $scope.collapsed = !$scope.collapsed;
                $scope.stripeRows = ($scope.collapsed) ? alwaysVisibleRows : visibleRows;
            };

        } else {
            $scope.stripeRows = visibleRows;
        }
    }

    $scope.$watchCollection(function () {
        return self.widget.fields.filter(function (field) {
            return field.isVisible && field.aclIsVisible;
        });
    }, function (fields) {
        buildStripeFields(fields);
    });
}