function LibLoginController(LibLoginService, LibUtilities, localStorageEngine, templateManager, eventAggregator, LibMessages,
                            ScreenService, metaDataFactory, WidgetsMetaData, ZhuAclController, ZhLogger, ZHPromise) {

    var loginUrlParams, loginUserId, loginUserName;

    var factoryExpiredSessionWidget = function() {
        return metaDataFactory.widgetFactory(WidgetsMetaData.getExpiredSession(), templateManager.container);
    };

    this.authenticate = function(title, theme, splashFavicon, image, product, productId,
                                 versionFile, testConnectionParams, language, settings, service) {
        var defer = ZHPromise.defer();
        var ip, bHash, setAclEventId;

        var setAclFunction = function() {
            setAcl(defer.promise, setAclEventId);
        };

        setAclEventId = eventAggregator.subscribe('onContainerOpen', setAclFunction);

        LibLoginService.getIpAddress().then(function(ipResponse) {
            if (ipResponse.dataset && ipResponse.dataset.ip) {
                ip    = ipResponse.dataset.ip.IP;
                bHash = ipResponse.dataset.ip.BHASH;
                var modulesVersionError = ipResponse.dataset.ip.MODULES_VERSION_ERROR;
                return ipResponseAndRequestToken(modulesVersionError, bHash);
            } else {
                return ZHPromise.reject(ipResponse);
            }
        }).then(function (tokenData) {
            loginUrlParams = LibLoginService.getUriParams(title, theme, splashFavicon, image, product,
                settings, language, productId, service, ip, bHash);
            return tokenResponseAndStartSession(tokenData, versionFile, productId);
        }).then(function (response) {
            return startSessionResponse(response);
        }).then(function (response) {
            defer.resolve(response);
        }).catch(function(error) {
            defer.reject(error);
            LibUtilities.showErrorMessage(error);
        });

        return defer.promise;
    };

    var ipResponseAndRequestToken = function(modulesVersionError, bHash) {
        if (modulesVersionError) {
            ZhLogger.warning(modulesVersionError);
        }
        return LibLoginService.requestToken(LibConfig.loginClientURL, LibConfig.loginURI, bHash);
    };

    var tokenResponseAndStartSession = function(tokenData, versionFile, productId) {
        if (tokenData.isError) {
            return ZHPromise.reject(tokenData.message);
        } else {
            if (tokenData && tokenData.length > 0) {
                localStorageEngine.setLocalVar('LIB_TOKEN', tokenData[0]);
                return LibLoginService.startSession(tokenData[0], versionFile, productId);
            } else {
                LibUtilities.redirectToLogin(loginUrlParams);
                return ZHPromise.when({});
            }
        }
    };

    var startSessionResponse = function(response) {
        if (response.isError) {
            return ZHPromise.reject(response.message);
        } else {
            if (!response.USER_ID) {
                localStorage.removeItem('IS_LOGGED');
                LibUtilities.redirectToLogin(loginUrlParams);
            } else if (response.SESSION_EXPIRED) {
                logoutExpiredSession(true);
            } else {
                localStorage.setItem('IS_LOGGED', true);
                loginUserId    = response.USER_ID;
                loginUserName  = response.USER_NAME;

                if(response.IS_PENDENT_MIGRATION) {
                    ScreenService.showMessage(ScreenService.i18n(LibMessages.ALE_MIGRATION_IS_PENDENT));
                }
            }

            return ZHPromise.when(response);
        }
    };

    var doLogout = function (params) {
        LibLoginService.logout().then(function () {
            localStorage.removeItem('IS_LOGGED');
            LibUtilities.redirectToLogin(params);
        }).catch(LibUtilities.showErrorMessage);
    };

    this.logout = function() {
        doLogout(loginUrlParams);
    };

    var setAcl = function (loginPromise, setAclEventId) {
        loginPromise.then(function(response) {
            if(!window.productApplyAcl) {
                ZhuAclController.setUserRules(response.ACCESS_PROFILE, window.projectId, response.USER_ID, response.ORGANIZATION_ID,
                    {
                        canConfigureAcl: response.ACL_ADMINISTRATOR,
                        rules          : response.ACL_ADMINISTRATOR ? [] : false,
                        defaultRule    : response.ACL_ADMINISTRATOR ? 'A' : false
                    }
                );
                templateManager.project.aclDefaultRule = 'A';
            }
            eventAggregator.unSubscribe('onContainerOpen', setAclEventId);
        }).catch(LibUtilities.showErrorMessage);
    };

    eventAggregator.subscribe("LibExpiredSession", function () {
        if(!templateManager.container.getActiveWidget().tekLibExpiredSession) {
            var expiredSessionWidget = factoryExpiredSessionWidget();
            expiredSessionWidget.attempts = 0;
            expiredSessionWidget.dataSource.data = [];
            expiredSessionWidget.newRow();
            expiredSessionWidget.currentRow.OPERATOR = loginUserId + ' - ' + loginUserName;
            ScreenService.openPopup(expiredSessionWidget);
            localStorageEngine.setLocalVar('LOGIN_EXPIRED_SESSION', true);

        }
    });

    var callDoLogout = function() {
        doLogout(loginUrlParams);
    };

    var logoutExpiredSession = function (afterLogin) {
        localStorageEngine.setLocalVar('LOGIN_EXPIRED_SESSION', false);
        var loadingMessage = afterLogin ? LibMessages.ERR_EXPIRED_SESSION_LOGIN : LibMessages.ERR_ATTEMPT_END;
        ScreenService.changeLoadingMessage(loadingMessage);
        ScreenService.showLoader();
        Util.doAsync(callDoLogout, 5000);
    };

    this.expiredSessionActionEventConfirm = function(args) {
        var expiredSessionWidget = args.owner.widget ? args.owner.widget : args.owner.field.widget;
        if(expiredSessionWidget.isValid()) {
            var password   = expiredSessionWidget.currentRow.PASSWORD;
            LibLoginService.validateUser(loginUserId, password)
                .then(function() {
                    localStorageEngine.setLocalVar('LOGIN_EXPIRED_SESSION', false);
                    ScreenService.closePopup();
                }).catch(function() {
                    if(expiredSessionWidget.attempts >= 2) {
                        ScreenService.showMessage(LibMessages.ERR_ATTEMPT_PARTIAL.replace('%1', ++expiredSessionWidget.attempts), 'ALERT');
                        logoutExpiredSession();
                    } else {
                        ScreenService.showMessage(LibMessages.ERR_ATTEMPT_PARTIAL.replace('%1', ++expiredSessionWidget.attempts), 'ALERT');
                    }
                });
        }
    };

    this.expiredSessionActionEventCancel = function() {
        localStorageEngine.setLocalVar('LOGIN_EXPIRED_SESSION', false);
        doLogout(loginUrlParams);
    };

    this.expiredSessionFieldOnKeyupPassword = function(args) {
        var key = window.event.keyCode;

        //13 = ENTER
        if(key == 13) {
            this.expiredSessionActionEventConfirm(args);
        }
    };

    this.downloadModulesVersionFile = function() {
        return LibLoginService.generateModulesVersionFile().then(function(response) {
            if(response && response.dataset) {
                var fileName = response.dataset.generatedFile[0];
                LibLoginService.downloadModulesVersionFile(fileName);
                return true;
            } else {
                LibUtilities.showErrorMessage(response);
                return false;
            }
        }).catch(LibUtilities.showErrorMessage);
    }
}

Configuration(function(ContextRegister){
    ContextRegister.register('LibLoginController', LibLoginController);
});